/*
**> In this example, we compare the Binary prices using Forward PDE solver and BS formula
**
*/

new;
library option,pgraph;
optionSet;

S0 = 100;
sigma = 0.20;
tau = 1;
b = 0.05;
r = 0.05;
p = 0;

proc tminBound(t,x);
  local PayOff;
  PayOff = EuropeanPayOff(S0,x);
  retp( PayOff );
endp;

proc xmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc DxminBound(t,x);
  retp( -ones(rows(x),1) );
endp;

proc aProc(t,x);
  retp( 0.5 * sigma^2 .* x^2 );
endp;

proc bProc(t,x);
  retp( (sigma^2 - b) .* x + sigma .* p .* x^2 );
endp;

proc cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local PayOff;
  PayOff = S0 .>= x;
  retp( PayOff );
endp;

proc DxminBound(t,x);
  retp( 0*ones(rows(x),1) );
endp;

proc xmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

xmin = 0; xmax = 200; Nx = 1001;
tmin = 0; tmax = tau; Nt = 1001;

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,0,&xmaxBound,&DxminBound,0);

call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"tmp",0.5);

x = PDE_readFile("tmp","x");
t = PDE_readFile("tmp","t");
tau = t[rows(t)];
U = PDE_readFile("tmp","t"|tau);
BCC = BinaryBS(S0,x,sigma,tau,b,r);

graphset;
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|3; _pcolor = 9|12; _plwidth = 10;
  _plegstr = "PDE\000BS";
  _plegctl = {2 5 5 5};
  xlabel("\216K");
  ylabel("\216BCC");
  xy(x,U~BCC);