/*
**> Call/Put on WorstOf Call/Put Option
** 
*/

new;
library option,pgraph;

cls;

let S0 = 100 100 100 100;            
let mu = 0.05 0.06 0.07 0.08;
let sigma = 0.15 0.25 0.20 0.30;                  // Implied Volatility

// Correlation Matrix between Asset returns

let rho = {1,
           0.5, 1,
           0.25, 0.15, 1,
           0.5,  0.5, 0.00, 1.00};
rho = xpnd(rho); 

// First, we have to correct the drift
r = 0.05;                                    // risk-free interest rate
mu = ones(rows(mu),1) * r;

// Caracteristics of the option
t = 1;                                       // 1 Fixing Date: 1 Year
tau = t[1];                                  // Maturity of the option = 1 Year
Notional = 1;                                // Notional of the option
K = 5.00;                                    // Strike of the WorstOf option = 5
fn call_payoff(S,K) = (S - K) .* (S .>= K);  // call option
fn put_payoff(S,K) = (K - S) .* (S .<= K);   // put option

Ns = 100000;  // Number of simulations
S = array_simulate_mGBM(S0,mu,sigma,rho,t,Ns);
S = mGBM_getDate(t,S,1)';

// Max(Call,Call,Put,Put)

K1 = 102; K2 = 102; K3 = 102; K4 = 95;

X = call_payoff(S[1,.],K1) |
    call_payoff(S[2,.],K2) |
    call_payoff(S[3,.],K3) |
    put_payoff(S[4,.],K4) ;

WorstOf = minc(X);

C = Notional * exp(-r*tau) * meanc( call_payoff(WorstOf,K) );
P = Notional * exp(-r*tau) * meanc( put_payoff(WorstOf,K) );

print ftos(C,"Call WorstOf C/C/C/P Option Price = %lf",3,2);
print ftos(P,"Put WorstOf C/C/C/P Option Price = %lf",3,2);