new;
library option,pgraph;
dlibrary option_tridiag; // Don't miss to download the tridiag solver or use the procedure OptionSet

cls;

S0 = 100;
K = 100;
sigma = 0.20;
tau = 0.5;
r = 0.05;
b = 0.02;

proc SigmaSurface(tau,K);
  retp( sigma + 0.*tau + 0.* K );
endp;

proc Bwrd_aProc(t,x);
  retp( 0.5*(sigma^2) .* (x^2) );
endp;

proc Fwrd_aProc(t,x);
  retp( 0.5*(SigmaSurface(t,x)^2) .* (x^2) );
endp;

proc Bwrd_bProc(t,x);
  retp( b .* x );
endp;

proc Fwrd_bProc(t,x);
  retp( - b .* x );
endp;

proc Bwrd_cProc(t,x);
  retp( r .* ones(rows(x),1) );
endp;

proc Frwd_cProc(t,x);
  retp( (r-b) .* ones(rows(x),1) );
endp;

proc dProc(t,x);
  retp( zeros(rows(x),1) );
endp;

proc Bwrd_tminBound(t,x);
  local PayOff;

  PayOff = EuropeanPayOff(x,K);

  retp( PayOff );
endp;

proc Fwrd_tminBound(t,x);
  local PayOff;

  PayOff = EuropeanPayOff(S0,x);

  retp( PayOff );
endp;

proc Bwrd_xminBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc Frwd_xmaxBound(t,x);
  retp( zeros(rows(x),1) );
endp;

proc Bwrd_DxmaxBound(t,x);
  retp( ones(rows(x),1) );
endp;

proc Fwrd_DxminBound(t,x);
  retp( -ones(rows(x),1) );
endp;

// Backward PDE

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;
__output = 1;

xmin = 0; xmax = 200; Nx = 1001;
tmin = 0; tmax = tau; Nt = 2500;

call PDE(&Bwrd_aProc,&Bwrd_bProc,&Bwrd_cProc,&dProc,0,&Bwrd_tminBound,&Bwrd_xminBound,0,0,&Bwrd_DxmaxBound);

fileName = "dupire-backward";
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

// Forward PDE

_PDE_SaveLastIter = 1;
_PDE_PrintIters = 50;
__output = 1;

xmin = 0; xmax = 200; Nx = 1001;
tmin = 0; tmax = tau; Nt = 2500;

call PDE(&Fwrd_aProc,&Fwrd_bProc,&Frwd_cProc,&dProc,0,&Fwrd_tminBound,0,&Frwd_xmaxBound,&Fwrd_DxminBound,0);

fileName = "dupire-forward";
call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,fileName,0.5);

// Comparison

fileName = "dupire-backward";

tau = PDE_readFile(fileName,"t");
x = PDE_readFile(fileName,"x");
U = PDE_readFile(fileName,"t"|tau);
e = x .> 10 .and x .< 190;
U1 = selif(U,e);
x1 = selif(x,e);
C1 = EuropeanBS(x1,K,sigma,tau,b,r);

fileName = "dupire-forward";

tau = PDE_readFile(fileName,"t");
x = PDE_readFile(fileName,"x");
U = PDE_readFile(fileName,"t"|tau);
e = x .> 10 .and x .< 160;
U2 = selif(U,e);
x2 = selif(x,e);
C2 = EuropeanBS(S0,x2,sigma,tau,b,r);

reldiff1 = 100 * (U1-C1)./C1;
reldiff2 = 100 * (U2-C2)./C2;

N = maxc(rows(x1)|rows(x2));
x = miss(zeros(N,2),0);
reldiff = miss(zeros(N,2),0);
x[1:rows(x1),1] = x1;
x[1:rows(x2),2] = x2;
reldiff[1:rows(x1),1] = reldiff1;
reldiff[1:rows(x2),2] = reldiff2;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; _plwidth = 10;
  xlabel("\216S]0[ or K");
  ylabel("\216Relative error (in %)");
  ytics(-0.005,0.010,0.0025,0);
  xtics(50,150,10,2);
  _plegstr = "Backward PDE\000Forward PDE";
  _plegctl = {2 5 6.5 1};
  graphprt("-c=1 -cf=dupire1.eps");
  xy(x,reldiff);
