new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

/*
**> Cox, Ingersoll et Ross [1995]
*/

kappa = 0.8; theta = 0.10; sigma = 0.5; lambda = 0.05;

proc muProc(t,x);
  retp(kappa*(theta-x));
endp;

proc sigmaProc(t,x);
  retp(sigma*sqrt(x));
endp;

proc lambdaProc(t,x);  
  retp(lambda*sqrt(x)/sigma);
endp;

proc rProc(t,x);
  retp(x);
endp;


/*
**> Building the PDE problem
**
**  We use the Neumann conditions
*/

proc aProc(t,x);
  retp( 0.5 * sigmaProc(t,x)^2 ); 
endp;


proc bProc(t,x);
  retp( muProc(t,x) - lambdaProc(t,x).*sigmaProc(t,x) );
endp;

proc cProc(t,x);
  retp( rProc(t,x) );
endp;

proc dProc(t,x);  
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  retp( ones(rows(x),1) );
endp;

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,0,0,0,0);


/*
**> Crank-Nicholson algorithm
*/

tmin = 0;
tmax = 5;
Nt = 101;
xmin = 0;
xmax = 1;
Nx = 51;

_pde_PrintIters = 3;     
_pde_SaveLastIter = 0;

call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);

t = PDE_readFile("pde","t");
x = PDE_readFile("pde","x");
U = PDE_readFile("pde",0);

call DeleteFile("pde.dat");

graphset;
  _pdate = ""; _pnum = 2; _pltype = 3|1|6|1|3; _pstype = 8|8|1|9|9; _plctrl = 75|75|0|75|75;
  fonts("simplex simgrma");
  title("\216Yield curve model CIR @[1995@]");
  xlabel("\216r]0[");
  ylabel("\216\202t\201");
  zlabel("\216P[c](t]0[)");
  _pview = 1|-6|5|5;
  surface(x',t,U);
