/*
**> In this example, we show how to use the ATM/Beta/Nu/Rho parametrization 
**  in place of the Alpha/Beta/Nu/Rho Calibration.
**
**  The idea is to calibrate the Alpha/Beta/Nu/Rho parametrization for
**  the actual smile and then use the relationship between alpha and ATM volatility.
**
*/

new;

#ifdef USE_OPTMUM
  library option,pgraph,optmum;
#else
  library option,pgraph;
#endif

optionSet;

__output = 0;

cls;

F0 = 0.05;
tau = 1;

let Strike = 0.04 0.05 0.06;
let ImpVol = 0.11 0.09 0.10;

beta = 0.5;
{alpha,beta,nu,rho} = SABR_Calibrate(F0,Strike,ImpVol,beta,tau,0);
print "---------------------------------------------------------";
print "Alpha/Beta/Nu/Rho Parametrization";
print "---------------------------------------------------------";
print alpha~beta~nu~rho; print;

CalVol = SABR_ImpVol(F0,Strike,alpha,beta,nu,rho,tau);
print "     Strike            ImpVol          CalVol";
print Strike~ImpVol~CalVol;
print;

print "---------------------------------------------------------";
print "ATM/Beta/Nu/Rho Parametrization";
print "---------------------------------------------------------";
ATM = 0.09;
alpha = SABR_Calibrate_ATM(F0,ATM,beta,nu,rho,tau);
print /flush "ATM = 9% "$+ftos(alpha,"---> alpha = %lf",3,5);
ATM = 0.10;
alpha = SABR_Calibrate_ATM(F0,ATM,beta,nu,rho,tau);
print /flush "ATM = 10% "$+ftos(alpha,"---> alpha = %lf",3,5);
print;

ImpVol1 = SABR_ImpVol_ATM(F0,Strike,ATM,beta,nu,rho,tau);
ATM = 0.10;
ImpVol2 = SABR_ImpVol_ATM(F0,Strike,ATM,beta,nu,rho,tau);
print "                       ATM = 9%  --->   ATM = 10%";
print "     Strike            CalVol           CalVol";
print Strike~ImpVol1~ImpVol2;