new;
library pf,pgraph;

cls; 

/* 
**> Sample the importance density
*/

proc (1) =  pf_importance_sampling(t,x,z);

  x = pf_prior_sampling(t,x);

  retp(x);
endp;

/* 
**> Sample the prior density
*/

proc (1) =  pf_prior_sampling(t,x);
  local nS,P,u,Ft,Qt;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc, Q:proc;

  nS = rows(x);
  P = cols(x);
  u = rndn(Ns,P);

  Ft = F(t,x);
  Qt = Q(t,x);  

  x = Ft + sqrt(Qt) * u;

  retp(x);
endp;


/* 
**> Evaluate the importance density function: q(x_{t}|x{t-1},z_{t})
*/

proc (1) = pf_importance_density(t,nx,x,z);
  local pdf;

  pdf = pf_prior_density(t,nx,x);

  retp(pdf);
endp;

/* 
**> Evaluate the prior density function: p(x_{t}|x{t-1})
**
** Remark: It is the same as the importance density function
*/

proc (1) = pf_prior_density(t,nx,x);
  local Ft,Qt,pdf;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc,Q:proc;

  Ft = F(t,x);
  Qt = Q(t,x);

  pdf = pdfn( (nx - Ft) ./ sqrt(Qt) ) ./ sqrt(Qt);

  retp(pdf);
endp;

/* 
**> Evaluate the likelihood function p(z_{t}|x_{t})
*/

proc (1) = pf_likelihood_density(t,nx,z);
  local Ht,Rt,pdf;

  local H,R;
  H = _pf_ht;
  R = _pf_Rt;
  local H:proc,R:proc;
  
  Ht = H(t,nx);
  Rt = R(t,nx);
  
  pdf = pdfn( (z - Ht) ./ sqrt(Rt) ) ./ sqrt(Rt);

  retp(pdf);
endp;

/* 
**> Sample the initial state vector
*/

proc (1) = pf_initial_sampling(nS);
  local x;
   x = 5*ones(nS,1);
  retp(x);
endp;

/*
**> State Space Model
*/

fn pf_ft(t,x) = x./2 + 25*x./(1 + x^2) + 8*cos(1.2*t);
fn pf_Qt(t,x) = 1;

fn pf_ht(t,x) = x^2 ./ 20;
fn pf_Rt(t,x) = 10;

/*
**> Main Test
**
*/

call Particle_Filter_Set(&pf_Ft,&pf_Qt,&pf_Ht,&pf_Rt,
                         &pf_importance_density,&pf_prior_density,&pf_likelihood_density,0,
                         &pf_importance_sampling,&pf_prior_sampling,&pf_initial_sampling);

/* Simulate the state and observed variables */

x0 = pf_initial_sampling(1);
nT = 100;
nR = 1;

nDates = nT;

/* Run the particle filter */

rndseed 123;
nRuns = 100;  // Number of runs
nS = 1000;  // Number of simulations
P = 1;      // Number of state variables
N = nS;    // Number of trajectories for the smoother
results_xt = zeros(nDates,nRuns);
results_zt = zeros(nDates,nRuns);

SIR_m = zeros(nDates,nRuns);
SIR_rmse = zeros(nRuns,1);

Smoothed_m = zeros(nDates,nRuns);
Smoothed_rmse = zeros(nRuns,1);

print "--------- RMSE -----------";
print "             PF     PS    ";

i = 1;
do until i > nRuns;


    {t,xt,zt} = Simulate_Particle_Filter(x0,nT,nR);          /* Simulating a SSM */
    results_xt[.,i] = xt;
    results_zt[.,i] = zt;
    
    {pf_xall,pf_w,pf_m,pf_cov} = SIR_Particle_Filter(zt,nS);             /* Running the SIR particle filter */     
    pf_x = array3_getmatrix(pf_xall,3,1);
    SIR_m[.,i] = pf_m;
    SIR_rmse[i] = sqrt(meanc((pf_m - xt)^2))';   
    
    {ps_xall,ps_w} = Particle_Smoother(pf_xall, pf_w,N);            /* Running the particle smoother */

    ps_x = array3_getmatrix(ps_xall,3,1);
    ps_m = sumr(ps_x .* ps_w);
    Smoothed_m[.,i] = ps_m;
    Smoothed_rmse[i] = sqrt(meanc((ps_m - xt)^2))';   
    
    str = ftos(i,"%f",8,0) $+
        
        ftos(SIR_rmse[i],"%f",8,3) $+
        ftos(Smoothed_rmse[i],"%f",8,3) ;
   print /flush str;
    i = i + 1;
endo;


print "---------------------------------------";

str = "mean:   " $+
      ftos(meanc(SIR_rmse),"%f",8,3) $+
      ftos(meanc(Smoothed_rmse),"%f",8,3) ;
print /flush str;

str = "std-err:" $+
      ftos(stdc(SIR_rmse),"%f",8,3) $+
      ftos(stdc(Smoothed_rmse),"%f",8,3) ;
print /flush str;


run_indx = 1;
do until run_indx > nRuns;

  graphset;
    window3d;
      _pdate = 0; _pnum = 2; _pframe = 0;
      _pnumht = 0.225; _paxht = 0.225; _ptitlht = 0.27;
      _plwidth = 10; _pltype = 6|3|1;
      _pcolor = 9|10|12;
      xlabel("\216t");

    setwind(1);
      _pcolor = 9;
      title("\216State values x]t[");
      xy(0,results_xt[.,run_indx]);

    setwind(2);
      _pcolor = 10;
      title("\216Measurements z]t[");
      xy(0,results_zt[.,run_indx]);
  
    setwind(3);
      _pcolor = 11|12|1|13; _pltype = 6|3|2|1;
      _plegstr = "PF\000PS"; 
      _plegctl = {2 5 9 4};
      title("\216Non-linear filtering");
      xy(0,SIR_m[.,run_indx]~Smoothed_m[.,run_indx]);

    endwind;

  call pause(0.2);

  run_indx = run_indx + 1;
endo;

pqgwin many;




data  = SIR_rmse~Smoothed_rmse;
d_rmse = zeros(256,2);
i = 1;
do until i > 2;
  {x_rmse,d_rmse[.,i]} = pf_Kernel(Data[.,i],0,10,256);
  i = i + 1;
endo;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 11|12|1|13; _pltype = 6|3|2|1;
  _plwidth = 10;
  title("\216pdf of RMSE");
  _plegstr = "PF\000PS"; 
  _plegctl = {2 5 7 5};
  xy(x_rmse,d_rmse);
