new;
library pf,pgraph;

cls; 

/* 
**> Sample the importance density
*/

proc (1) =  pf_importance_sampling(t,x,z);

  x = pf_prior_sampling(t,x);

  retp(x);
endp;

/* 
**> Sample the prior density
*/

proc (1) =  pf_prior_sampling(t,x);
  local nS,P,u,Ft,Qt;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc, Q:proc;

  nS = rows(x);
  P = cols(x);
  u = rndn(Ns,P);

  Ft = F(t,x);
  Qt = Q(t,x);  

  x = Ft + sqrt(Qt) * u;

  retp(x);
endp;


/* 
**> Evaluate the importance density function: q(x_{t}|x{t-1},z_{t})
*/

proc (1) = pf_importance_density(t,nx,x,z);
  local pdf;

  pdf = pf_prior_density(t,nx,x);

  retp(pdf);
endp;

/* 
**> Evaluate the prior density function: p(x_{t}|x{t-1})
**
** Remark: It is the same as the importance density function
*/

proc (1) = pf_prior_density(t,nx,x);
  local Ft,Qt,pdf;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc,Q:proc;

  Ft = F(t,x);
  Qt = Q(t,x);

  pdf = pdfn( (nx - Ft) ./ sqrt(Qt) ) ./ sqrt(Qt);

  retp(pdf);
endp;

/* 
**> Evaluate the likelihood function p(z_{t}|x_{t})
*/

proc (1) = pf_likelihood_density(t,nx,z);
  local Ht,Rt,pdf;

  local H,R;
  H = _pf_ht;
  R = _pf_Rt;
  local H:proc,R:proc;
  
  Ht = H(t,nx);
  Rt = R(t,nx);
  
  pdf = pdfn( (z - Ht) ./ sqrt(Rt) ) ./ sqrt(Rt);

  retp(pdf);
endp;

/* 
**> Sample the initial state vector
*/

proc (1) = pf_initial_sampling(nS);
  local x;
   x = 5*ones(nS,1);
  retp(x);
endp;

/*
**> State Space Model
*/

fn pf_ft(t,x) = x./2 + 25*x./(1 + x^2) + 8*cos(1.2*t);
fn pf_Qt(t,x) = 1;

fn pf_ht(t,x) = x^2 ./ 20;
fn pf_Rt(t,x) = 10;

/*
**> Main Test
**
*/

call Particle_Filter_Set(&pf_Ft,&pf_Qt,&pf_Ht,&pf_Rt,
                         &pf_importance_density,&pf_prior_density,&pf_likelihood_density,0,
                         &pf_importance_sampling,&pf_prior_sampling,&pf_initial_sampling);

/* Simulate the state and observed variables */

x0 = pf_initial_sampling(1);
nT = 100;
nR = 1;

nDates = nT;

/* Run the particle filter */

rndseed 123;

nS = 10|50|500;  // Number of particles
nRuns = 100;     // Number of runs of the MC
P = 1;           // Number of state variables

rmse = zeros(nRuns,rows(nS));

i = 1;
do until i > nRuns;

  {t,xt,zt} = Simulate_Particle_Filter(x0,nT,nR);     /* Simulating a SSM */

  {x,w,m,cov} = SIR_Particle_Filter(zt,nS[1]);               
  rmse[i,1] = sqrt(meanc((m - xt)^2))';   

  {x,w,m,cov} = SIR_Particle_Filter(zt,nS[2]);               
  rmse[i,2] = sqrt(meanc((m - xt)^2))';   

  {x,w,m,cov} = SIR_Particle_Filter(zt,nS[3]);               
  rmse[i,3] = sqrt(meanc((m - xt)^2))';  

  print /flush i; 

  i = i + 1;   
endo; 

d_rmse = zeros(256,rows(nS));
i = 1;
do until i > rows(nS);
  {x_rmse,d_rmse[.,i]} = pf_Kernel(rmse[.,i],0,15,256);
  i = i + 1;
endo;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 1|2|12; _pltype = 6|3|1;
  _plwidth = 10;
  title("\216SIR Algorithm");
  xlabel("\216RMSE");
  xtics(0,12,2,0);
  _plegstr = "10 particles\00050 particles\000500 particles"; 
  _plegctl = {2 4 5.25 5};
  graphprt("-c=1 -cf=example3.eps");    
  xy(x_rmse,d_rmse);