new;
library pf,pgraph;

cls; 

/* 
**> Sample the importance density
*/

proc (1) =  pf_importance_sampling(t,x,z);

  x = pf_prior_sampling(t,x);

  retp(x);
endp;

/* 
**> Sample the prior density
*/

proc (1) =  pf_prior_sampling(t,x);
  local nS,P,u,Ft,Qt;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc, Q:proc;

  nS = rows(x);
  P = cols(x);
  u = rndn(Ns,P);

  Ft = F(t,x);
  Qt = Q(t,x);  

  x = Ft + sqrt(Qt) * u;

  retp(x);
endp;


/* 
**> Evaluate the importance density function: q(x_{t}|x{t-1},z_{t})
*/

proc (1) = pf_importance_density(t,nx,x,z);
  local pdf;

  pdf = pf_prior_density(t,nx,x);

  retp(pdf);
endp;

/* 
**> Evaluate the prior density function: p(x_{t}|x{t-1})
**
** Remark: It is the same as the importance density function
*/

proc (1) = pf_prior_density(t,nx,x);
  local Ft,Qt,pdf;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc,Q:proc;

  Ft = F(t,x);
  Qt = Q(t,x);

  pdf = pdfn( (nx - Ft) ./ sqrt(Qt) ) ./ sqrt(Qt);

  retp(pdf);
endp;

/* 
**> Evaluate the likelihood function p(z_{t}|x_{t})
*/

proc (1) = pf_likelihood_density(t,nx,z);
  local Ht,Rt,pdf;

  local H,R;
  H = _pf_ht;
  R = _pf_Rt;
  local H:proc,R:proc;
  
  Ht = H(t,nx);
  Rt = R(t,nx);
  
  pdf = pdfn( (z - Ht) ./ sqrt(Rt) ) ./ sqrt(Rt);

  retp(pdf);
endp;

/* 
**> Sample the initial state vector
*/

proc (1) = pf_initial_sampling(nS);
  local x;
   x = 5*ones(nS,1);
  retp(x);
endp;

/*
**> State Space Model
*/

fn pf_ft(t,x) = x./2 + 25*x./(1 + x^2) + 8*cos(1.2*t);
fn pf_Qt(t,x) = 1;

fn pf_ht(t,x) = x^2 ./ 20;
fn pf_Rt(t,x) = 10;

/*
**> Main Test
**
*/

call Particle_Filter_Set(&pf_Ft,&pf_Qt,&pf_Ht,&pf_Rt,
                         &pf_importance_density,&pf_prior_density,&pf_likelihood_density,0,
                         &pf_importance_sampling,&pf_prior_sampling,&pf_initial_sampling);

/* Simulate the state and observed variables */

x0 = pf_initial_sampling(1);
nT = 100;
nR = 1;

nDates = nT;

/* Run the particle filter */

nS = 100;   // Number of particles
P = 1;       // Number of state variables

{t,xt,zt} = Simulate_Particle_Filter(x0,nT,nR);     /* Simulating a SSM */

_pf_Save_Particles = 1;  // Save particles

{x,w,m,cov} = SIS_Particle_Filter(zt,nS);                /* Running the SIS particle filter */     
x = array3_getmatrix(x,3,1);
SIS_m = m;
SIS_x = x;

{x,w,m,cov} = Generic_Particle_Filter(zt,nS);             /* Running the generalized particle filter */     
x = array3_getmatrix(x,3,1);
PF_m = m;
PF_x = x;

{x,w,m,cov} = SIR_Particle_Filter(zt,nS);             /* Running the SIR particle filter */     
x = array3_getmatrix(x,3,1);
SIR_m = m;
SIR_x = x;

{x,w,m,cov} = Regularized_Particle_Filter(zt,nS);  /* Running the regularised particle filter */   
x = array3_getmatrix(x,3,1);
RPF_m = m;
RPF_x = x;
RPF_w = w;

pqgwin many;

graphset;
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);

  _pdate = ""; _pnum = 2; _pframe = 0;
  ytics(-25,25,5,2);  

  setwind(1);
    _plctrl = -1; _pstype = 4; _pcolor = 1; _psymsiz = 2;
    xy(t,RPF_x);
  
  setwind(2);
    _plctrl = 0; _pcolor = 12|10; _plwidth = 10; _pltype = 6|3;
    xy(t,xt~RPF_m);

    graphprt("-c=1 -cf=example4-1.eps");

  endwind;

n = 50;
x0 = seqa(-25,50/n,n+1);
c = (x0 + lag1(x0))/2;
c[1] = -25;
nC = rows(c);
frq = zeros(nC,nT);

x = RPF_x';
w = RPF_w';

i = 1;
do until i > nT;
  Frq[.,i] = countwts(x[.,i],c,w[.,i]);
  i = i + 1;
endo;

t = ones(nC,1) .* t';
x0 = x0 .* ones(1,nT);
data = vec(t)~vec(x0)~vec(Frq);
data = delif(data,data[.,3] .== 0);
nData = rows(data);
e = ones(nData,1);
psym = data[.,1 2]~e*8~10*data[.,3]~e.*(0~1~10);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  ytics(-25,25,5,2);  
  _pcolor = 12|10; _plwidth = 10; _pltype = 6|3;
  _psym = psym;
  graphprt("-c=1 -cf=example4-2.eps");
  xy(0,xt~RPF_m);

