new;
library qam,optmum,pgraph;

declare external epsilon;
declare external omega;
declare external _gamma;
declare external e;
declare external m;

proc duffing(t,z);
  local x,y,d;

  x = z[1];
  y = z[2];

  d = (y                                              )    |
      (x - x^3 - epsilon*y + _gamma*cos(omega*t)      )    ;

  retp(d);
endp;

proc vdp(t,z);
  local x,y,dz;

  x = z[1];
  y = z[2];

  dz = -y                    |
       x -e*(y^3/3 - m*y)    ;

  retp(dz);
endp;

let z0 = -1 1;
epsilon = 0.22; _gamma = 0.3; omega = 1;
{t1,z1,dz1} = ODE_Runge_Kutta(&duffing,z0,0,50,100);
{t,z,dz} = ODE_Runge_Kutta(&duffing,z0,0,5000,10000);
z0 = z[10000,.]';
{t2,z2,dz2} = ODE_Runge_Kutta(&duffing,z0,0,500,10000);

e = 1; m = 0.5;
z0 = zeros(2,1) + 0.00001;
{t3,z3,dz3} = ODE_Runge_Kutta(&vdp,z0,0,100,1000);
e = 1; m = 1.5;
z0 = zeros(2,1) + 0.00001;
{t,z,dz} = ODE_Runge_Kutta(&vdp,z0,0,100,1000);
z0 = z[1000,.]';
{t4,z4,dz4} = ODE_Runge_Kutta(&vdp,z0,0,100,1000);

graphset;
  begwind;
  window(2,2,1);

    _pnum = 2; _pdate = ""; _pframe = 0;
    _pltype = 6|3; _plwidth = 8;
    _ptitlht = 0.25; _pnumht = 0.20; _paxht = 0.25;
    fonts("simplex simgrma");

  setwind(1);
    _pcolor = 1|5;    
    title("\216Duffing\L\216\202e\201 = 0,22 - \202g\201 = 0,3 - \202w\201 = 1");
    xlabel("\216t");
    _plegstr = "x(t)\000y(t)";
    _plegctl = {2 5 7 1.5};
    xtics(0,50,10,2);
    ytics(-1.5,1.5,0.5,2);
    xy(t1,z1);

  _plegctl = 0;

  setwind(2);
    _pcolor = 2;    
    xlabel("\216x(t)");
    ylabel("\216y(t)");
    xtics(-1.5,1.5,0.5,2);
    ytics(-1.5,1.5,0.5,2);
    xy(z2[.,1],z2[.,2]);

  setwind(3);
    _pcolor = 3;    
    title("\216Van der Pol\L\216e = 1 - m = 0,5");
    xtics(-1.5,1.5,0.5,2);
    ytics(-1.5,1.5,0.5,2);
    xy(z3[.,1],z3[.,2]);

  setwind(4);
    _pcolor = 4;    
    title("\216Van der Pol\L\216e = 1 - m = 1,5");
    xtics(-3,3,1,2);
    ytics(-3,3,1,2);
    xy(z4[.,1],z4[.,2]);

   graphprt("-c=1 -cf=approxim35.eps");

  endwind;


