new;
library qam,optmum,pgraph;

declare external sigma,beta,R;
declare external a,b,c,d;

proc lorenz(t,u);
  local x,y,z,dx,dy,dz,du;

  x = u[1]; y = u[2]; z = u[3];

  dx = sigma*(y-x);
  dy = -x*z + R*x - y;
  dz = x*y - beta*z;

  du = dx|dy|dz;

  retp(du);
endp;

proc lvg(t,z);
  local x,y,dz;

  x = z[1];
  y = z[2];

  dz = a*x - b*x*y           |
       -c*y + d*x*y          ;

  retp(dz);
endp;

proc rossler(t,u);
  local x,y,z,dx,dy,dz,du;

  x = u[1]; y = u[2]; z = u[3];

  dx = -(y+z);
  dy = x + a*y;
  dz = b + x*z -c*z;

  du = dx|dy|dz;

  retp(du);
endp;

sigma = 10; beta = 8/3; R = 28;

u0 = 1|1|1;
{t,u,du} = ODE_Runge_Kutta(&lorenz,u0,0,5,500);
u0 = u[500,.]';
{t1,u1,du1} = ODE_Runge_Kutta(&lorenz,u0,0,20,1000);

a = 0.2; b = 0.2; c = 5.25;
u0 = 0|1|1;
{t,u,du} = ODE_Runge_Kutta(&rossler,u0,0,100,500);
u0 = u[500,.]';
{t2,u2,du2} = ODE_Runge_Kutta(&rossler,u0,0,100,2000);

a = 4; b = a; c = a; d = a;
x = {};
y = {};
i = 0.1;
do until i > 1.5;
  z0 = i|i;
  {t,z,dz} = ODE_Runge_Kutta(&lvg,z0,0,3,500);
  x = x~z[.,1];
  y = y~z[.,2];
  i = i + 0.2;
endo;
x3 = x; y3 = y;

graphset;
  window3d;

    _pnum = 2; _pdate = ""; _pframe = 0;
    _pltype = 6|3; _plwidth = 5; _pcolor = 1;
    _ptitlht = 0.25; _pnumht = 0.20; _paxht = 0.25;
    fonts("simplex simgrma");
    xlabel("\216x(t)");
    ylabel("\216y(t)");
    zlabel("\216z(t)");

  setwind(1);
    _pcolor = 1;
    title("\216Lorenz\L\216\202s\201 = 10 - \202b\201 = 8/3 - R = 28");
    xtics(-20,20,10,0);
    ytics(-20,20,10,0);
    ztics(0,50,10,0);
    xyz(u1[.,1],u1[.,2],u1[.,3]);

  setwind(2);
    _pcolor = 2;
    title("\216Rossler\L\216a = b = 0,2 - c = 5,25");
    xtics(-10,15,5,0);
    ytics(-10,10,5,0);
    ztics(0,20,5,0);
    xyz(u2[.,1],u2[.,2],u2[.,3]);

  setwind(3);
    _pcolor = 1|2|3|4|5|6;
    title("\216Lotka-Volterra-Godwin\L\216a = b = c = d = 4");
    xtics(0,6,1,0);
    ytics(0,6,1,0);
    xy(x3,y3);

   graphprt("-c=1 -cf=approxim37.eps");

  endwind;

