new;
library option,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

declare matrix Use_Fokker_Planck,Use_Density;

_fcmptol = 1e-12;

mu = 0.10;
sigma = 0.20;

declare matrix _x_end,x_begin;

proc aProc(t,x);
  retp( 0.5*(sigma^2) * ones(rows(x),1) ); 
endp;

proc bProc(t,x);
  if Use_Fokker_Planck;
    retp( -mu );
  else;
    retp( mu );
  endif;
endp;

proc cProc(t,x);
  retp( 0 * ones(rows(x),1) );
endp;

proc dProc(t,x);  
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local cn;

  save x;
  if Use_Fokker_Planck;
    cn = ones(rows(x),1) .* dotfeq(x,x_begin);
    if sumc(cn) == 0;
      errorlog "error"; 
      end;
    endif;

  else;
    if Use_Density;
      cn = ones(rows(x),1) .* dotfeq(x,_x_end);
      if sumc(cn) == 0;
        errorlog "error"; 
      endif;
    else;
      cn = ones(rows(x),1) .* (x .<= _x_end);
    endif;
  endif;

  retp(cn);
endp;

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,0,0,0,0);

tmin = 0;
tmax = 0.5;
Nt = 101;
xmin = -1;
xmax = 1;
Nx = 201;

_pde_PrintIters = 0;
_pde_SaveLastIter = 1;

x_end = seqa(-1,0.01,201);
nx = rows(x_end);

x_begin = 0;

cdf = cdfn((x_end - x_begin - mu*tmax)/(sigma*sqrt(tmax)));
pdf = pdfn((x_end - x_begin - mu*tmax)/(sigma*sqrt(tmax)))/(sigma*sqrt(tmax));

Use_Fokker_Planck = 1;

call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
t = PDE_readFile("pde","t");
x = PDE_readFile("pde","x");
U = PDE_readFile("pde","t"|t);
sol_Fokker_Planck = u;

Use_Fokker_Planck = 0;
Use_Density = 1;

sol = zeros(nx,1);

i = 1;
do until i > nx;

  _x_end = x_end[i];

  call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  U = PDE_readFile("pde","t"|t);
  sol[i] = u[101];
  
  i = i + 1;
endo;

sol_Feynmann_Kac = sol;
dx = (xmax - xmin)/(Nx - 1);

sol_Fokker_Planck = sol_Fokker_Planck/dx;
sol_Feynmann_Kac = sol_Feynmann_Kac/dx;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 1|3; _plwidth = 10; _pcolor = 1|2;
  xy(x,pdf~sol_Feynmann_Kac~sol_Fokker_Planck);
