new;
library qam,option,pgraph,optmum;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

declare matrix Use_Fokker_Planck,Use_Density;

Use_Density = 1;

_fcmptol = 1e-12;

a = 1.00;
b = 0.10;
sigma = 0.20;

declare matrix _x_end,x_begin;

proc aProc(t,x);
  retp( 0.5*(sigma^2) .* ones(rows(x),1) ); 
endp;

proc bProc(t,x);
  local mu_x;

  mu_x = a * ( b - x);

  if Use_Fokker_Planck;
    retp( - mu_x );
  else;
    retp( mu_x );
  endif;
endp;

proc cProc(t,x);
  if Use_Fokker_Planck;
    retp( -a .* ones(rows(x),1) );
  else;
    retp( 0 * ones(rows(x),1) );
  endif;
endp;

proc dProc(t,x);  
  retp( zeros(rows(x),1) );
endp;

proc tminBound(t,x);
  local cn;

  if Use_Fokker_Planck;
    cn = ones(rows(x),1) .* dotfeq(x,x_begin);
    if sumc(cn) /= 1;
      errorlog "error"; 
      end;
    endif;
  else;
    if Use_Density;
      cn = ones(rows(x),1) .* dotfeq(x,_x_end);
      if sumc(cn) == 0;
        errorlog "error"; 
      endif;
    else;
      cn = ones(rows(x),1) .* (x .<= _x_end);
    endif;
  endif;

  retp(cn);
endp;

call PDE(&aProc,&bProc,&cProc,&dProc,0,
         &tminBound,0,0,0,0);

tmin = 0;
tmax = 1;
Nt = 101;
xmin = -1;
xmax = 1;
Nx = 201;

_pde_PrintIters = 0;
_pde_SaveLastIter = 1;

x_end = seqa(-1,0.01,201);
nx = rows(x_end);

x_begin = 0;

vol = (sigma^2)*(1 - exp(-2*a*tmax))/(2*a);
vol = sqrt(vol);
pdf = pdfn( (x_end - (x_begin * exp(-a*tmax) + b * (1 - exp(-a*tmax)))) /vol) / vol;

Use_Fokker_Planck = 1;

call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",1);
t = PDE_readFile("pde","t");
x = PDE_readFile("pde","x");
U = PDE_readFile("pde","t"|t);
sol_Fokker_Planck = u;

Use_Fokker_Planck = 0;
Use_Density = 1;

sol = zeros(nx,1);

i = 1;
do until i > nx;

  _x_end = x_end[i];

  call PDE_solve(tmin,tmax,Nt,xmin,xmax,Nx,"pde",0.5);
  t = PDE_readFile("pde","t");
  x = PDE_readFile("pde","x");
  U = PDE_readFile("pde","t"|t);
  sol[i] = u[101];
  
  i = i + 1;
endo;

dx = (xmax - xmin)/(Nx - 1);
sol_Fokker_Planck = sol_Fokker_Planck/dx;

if Use_Density;
  sol_Feynmann_Kac = sol/dx;
else;
  sol_Feynmann_Kac = (sol - lag1(sol)) / dx;
endif;

graphset;
  window3u;

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; 
  _paxht = 0.16; _pnumht = 0.16; _ptitlht = 0.275;
  xtics(-0.5,0.5,0.10,2);  

  setwind(1);
    _pcolor = 1;
    title("\216Solution analytique");
    xy(x,pdf);

  setwind(2);
    _pcolor = 2;
    title("\216Feynman-Kac");
    xy(x,sol_Feynmann_Kac);

  setwind(3);
    _pcolor = 12;
    title("\216Fokker-Planck");
    xy(x,sol_Fokker_Planck);

    graphprt("-c=1 -cf=approxim53.eps");

  endwind;