new;
library pde2d,pgraph;

dlibrary option_tridiag; // Don't miss to download the tridiag solver

cls;

_fcmptol = 1e-12;

mu = 0.05; 
a = 0.50;
b = 0.20^2;
sigma = 0.10;
rho = -0.75;

declare matrix x_0,y_0;

proc aProc(t,x,y);
  retp( 0.5 * x^2 .* y );
endp;

proc bProc(t,x,y);
  retp( 0.5 * rho * sigma * x .* y );
endp;

proc cProc(t,x,y);
  retp( 0.5 * sigma^2 .* y .* ones(rows(x),1) );
endp;

proc dProc(t,x,y);
  retp( (2 * y + rho * sigma - mu) .* x );
endp;

proc eProc(t,x,y);
  retp( (sigma^2 + rho * sigma .* y - a * (b - y)) );
endp;

proc fProc(t,x,y);
  retp( (mu - a - y - rho*sigma) .* ones(rows(x),cols(y)) );
endp;

proc gProc(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc tminBound(t,x,y);
  local cn;
  cn = ones(rows(x),cols(y)) .* dotfeq(x,x_0) .* dotfeq(y,y_0);
  if sumc(sumc(cn)) /= 1;
    errorlog "error"; 
    end;
  endif;
  retp( cn );
endp;

proc DxminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DxmaxBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DyminBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

proc DymaxBound(t,x,y);
  retp( zeros(rows(x),cols(y)) );
endp;

call PDE2D(&aProc,&bProc,&cProc,&dProc,&eProc,&fProc,&gProc,0,
                  &tminBound,0,0,0,0,
                  &DxminBound,&DxmaxBound,&DyminBound,&DymaxBound);

xmin = 50; xmax = 170; Nx = 121;
ymin = 0.001; ymax = 0.10; Ny = 100;
tmin = 0; tmax = 0.51; Nt = 51;

x_0 = 100; y_0 = 0.05;

_Hopscotch_SaveLastIter = 1;
_Hopscotch_PrintIters = 1;
_Hopscotch_Methods = {1, 0.5};
call Hopscotch(tmin,tmax,Nt,xmin,xmax,Nx,ymin,ymax,Ny,"tmp");

x = ExtractSolution("tmp","x");
y = ExtractSolution("tmp","y");
tmax = ExtractSolution("tmp","t");
sol = ExtractSolution("tmp","t"|tmax);

dx = (xmax - xmin)/(Nx - 1);
dy = (ymax - ymin)/(Ny - 1);
sol_Fokker_Planck = sol/dx/dy;

graphset;
  surface(x',y',sol_Fokker_Planck');
