new;
library qam,pgraph,option,optmum;

rndseed 12345678;

K = 5;
S10 = 100; 
S20 = 100;
sigma1 = 0.20; 
sigma2 = 0.20; 
Tau = 1;
r = 0.05; 
rho = 0.50;
Ns = 1000;

Nr = 10000;
S1 = zeros(Ns,Nr);
S2 = S1; S1_av = S1; S2_av = S2;

i = 1;
do until i > Nr;
  {x1,x2,x1_av,x2_av} = simulate_GBM2_av(S10,S20,r,r,sigma1,sigma2,rho,tau,Ns);
  S1[.,i] = x1';
  S2[.,i] = x2';
  S1_av[.,i] = x1_av';
  S2_av[.,i] = x2_av';
  i = i + 1;
endo;

G = _max_(0,S1 - S2 - K);
G_av = _max_(0,S1_av - S2_av - K);

C = exp(-r*tau) * meanc(G);
C_av = exp(-r*tau) * meanc(0.5*(G+G_av));

x = seqa(4.5,2.5/255,256);
{d1,F} = regKernelDensity(C,x);
{d2,F} = regKernelDensity(C_av,x);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|3; _pcolor = 1|2; _plwidth = 10;
  xtics(4.5,7,0.5,2);
  ytics(0,2.2,0.2,2);
  _plegstr = "MC\000MC-AV";
  _plegctl = {2 5 6 5};
  _pline = 1~6~5.8198~0~5.8198~2.5~1~0~10;
  graphprt("-c=1 -cf=mc15.eps");
  xy(x,d1~d2);

/*
**> Simulation of a 2D Geometric Brownian Motion with antithetic variates
**
** x1 : vector Nt x Ns
** x2 : vector Nt x Ns
*/

proc (4) = simulate_GBM2_av(x01,x02,mu1,mu2,sigma1,sigma2,rho,t,Ns);
  local Nt,x1,x2,k1,k2,sqrt_k,u1,u2,i;
  local rho2,sqrt_dt,dt;
  local eta1,eta2,x1_av,x2_av,x01_av,x02_av;

  Nt = rows(t);

  x1 = zeros(Nt,Ns);
  x2 = zeros(Nt,Ns);

  x1_av = zeros(Nt,Ns);
  x2_av = zeros(Nt,Ns);

  x01 = x01 .* ones(1,Ns);
  x02 = x02 .* ones(1,Ns);

  x01_av = x01;
  x02_av = x02;

  rho2 = rho^2;

  i = 1;
  do until i > Nt;
    if i == 1;
      dt = t[1];
    else;
      dt = t[i] - t[i-1];
    endif;

    k1 = (mu1-0.5*sigma1^2)*dt;
    k2 = (mu2-0.5*sigma2^2)*dt;

    sqrt_dt = sqrt(dt);

    eta1 = rndn(1,Ns);
    eta2 = rndn(1,Ns);
    u1 = eta1;
    u2 = rho .* u1 + sqrt(1-rho2) * eta2;
    x01 = x01 .* exp(k1 + sigma1 .* sqrt_dt .* u1);
    x02 = x02 .* exp(k2 + sigma2 .* sqrt_dt .* u2);

    eta1 = -eta1; 
    eta2 = -eta2; 
    u1 = eta1;
    u2 = rho .* u1 + sqrt(1-rho2) * eta2;
    x01_av = x01_av .* exp(k1 + sigma1 .* sqrt_dt .* u1);
    x02_av = x02_av .* exp(k2 + sigma2 .* sqrt_dt .* u2);

    x1[i,.] = x01;
    x2[i,.] = x02;

    x1_av[i,.] = x01_av;
    x2_av[i,.] = x02_av;

    i = i + 1;
  endo;

  retp(x1,x2,x1_av,x2_av);
endp;
