new;
library qam,optmum,pgraph;

declare matrix a,b,c,d,e,f,T_end,nT;

proc (1) = ode(t,greeks);
  local alpha,beta,gamma_,dalpha,dbeta,dgamma,dgreeks;

  alpha = greeks[1];
  beta = greeks[2];
  gamma_ = greeks[3];

  dalpha = 2*(-e*alpha - 0.5*a + 0.5*(f^2)/b*alpha^2);
  dbeta = -beta*(e - (f^2)/b*alpha);
  dgamma = -0.5*(f^2)/b*(beta^2);
  
  dgreeks = dalpha|dbeta|dgamma;

  retp(dgreeks);
endp;

proc Terminal_Value(greek_0);
  local t,greeks,dgreeks,greek_T,u;

  {t,greeks,dgreeks} = ODE_Runge_Kutta(&ode,greek_0,0,T_end,nT);

  greek_T = greeks[nT,.]';
  u = greek_T - (d|c|0);
  retp(sumc(u^2));
endp;  

proc (2) = Solve_HJB(x,greeks);
  local alpha,beta,gamma_,J,dJ,v;

  alpha = greeks[.,1];
  beta = greeks[.,2];
  gamma_ = greeks[.,3];

  J = -0.5 * alpha .* (x^2) + beta .* x + gamma_;
  dJ = -alpha .* x + beta;
  v = dJ * f / b;

  retp(J,v);
endp;

proc (2) = Analytical_Solution(t,x,T_end);
  local omega1,omega2,K1,k2,J,v;

  omega1 = b ./ (f^2) .* (e + sqrt(e^2  + (f^2) .*a ./b));
  omega2 = b ./ (f^2) .* (e - sqrt(e^2  + (f^2) .*a ./b));

  K1 = 0.5 .* (f^2) ./ b .* (omega1 - omega2);
  K2 = (d - omega2) ./ (d - omega1);
 
  J = -0.5 .* omega1 .*(x^2) + 0.5 .* (c^2) ./ (d-omega1) - 
      (omega1 - omega2) .* ( (x - c .* exp(K1 .* (T_end - t)) ./ (d - omega1) )^2) ./
      ( 2 * (K2 .* exp(2 * K1 .* (T_end-t)) - 1.0) );
  
  v = -f ./ b .* ( omega1 .* x + (omega1 - omega2) .* (x - c .* exp(K1 .* (T_end - t) ) ./ (d - omega1)) ./
      (K2 .* exp(2 * K1 .* (T_end-t)) - 1.0) );

  retp(J,v);
endp;


T = 2; nT = 201;
a = 2; b = 1.5; c = 1; d = 1; f = 1.2;
greek_0 = 0|0|0;

{greek_0,fmin,grd,retcode} = optmum(&Terminal_Value,greek_0);
{t,greeks,dgreeks} = ODE_Runge_Kutta(&ode,greek_0,0,T,nT);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1; _plwidth = 10; _pcolor = 1|2|3|4;
  xy(t,greeks);

x = 2;

{J1,v1} =  Analytical_Solution(t,x,T_end);
{J2,v2} =  Solve_HJB(x,greeks);

t0 = 0;
greeks_t = greeks[1,.];
x = seqa(-5,10/100,101);
{J3,v3} =  Analytical_Solution(t0,x,T_end);
{J4,v4} =  Solve_HJB(x,greeks_t);

graphset;
  begwind;
  window(2,2,1);
  _pwindmx = _pwindmx | _pwindmx[4,.]; 

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1; _plwidth = 10; _pcolor = 1|2|3|4;
  _paxht = 0.25; _pnumht = 0.20; _ptitlht = 0.30;

  xtics(0,2,0.5,2);
  xlabel("\216t");
  setwind(1);
    title("\216J(t,2)");
    ytics(0,3.5,0.5,2);
    xy(t,J1~J2);

  setwind(2);
    title("\216v[*](t,2)");
    ytics(-1,3,1,2);
    xy(t,v1~v2);

  xtics(-5,5,2.5,2);
  xlabel("\216x");

  setwind(3);
    title("\216J(0,x)");
    ytics(-15,5,5,2);
    xy(x,J3~J4);

  setwind(4);
    title("\216v[*](0,x)");
    ytics(-5,5,2.5,2);
    _plegstr = "Solution analytique\000Solution numerique";
    _plegctl = {2 5.5 2 1.5};
    xy(x,v3~v4);

    _plegctl = 0;

  setwind(5);
    title("\216v[*](0,x)");
    ytics(-5,5,2.5,2);
    _pmsgstr = "-";
    _pmsgctl = 5.14~1.85~0.07~45~2~0~5 ;
    xy(x,v3~v4);

    graphprt("-c=1 -cf=bellman3.eps");

  endwind;

