new;
library qam,optmum,pgraph;

datFileName = "..\\data\\qam_spx_daily_option";

Data = loadd(datFileName);
vnames = getname(datFileName);

let selected_names = "date" "SPX_TR" "LIBOR";
indx = indcv(selected_names,vnames);
data = data[.,indx];

data = selif(data, (data[.,1] .>= 19900101) .and (data[.,1] .<= 20091231));
nDates = rows(data);

Dates = Data[.,1];
SPX = Data[.,2];
SPX = 100 * SPX / SPX[1];
R_Libor = Data[.,3]/100;
dt = etdays2(lag1(dates),dates)/365;
Libor = 1 + lag1(R_Libor) .* dt;
Libor[1] = 1.0;
Libor = 100 * cumprodc(Libor);
R_Libor = Libor ./ lag1(Libor) - 1.0;
R_Libor[1] = 0;

R_SPX = SPX ./ lag1(SPX) - 1.0;
R_SPX[1] = 0;

lambda = 0.97;
sigma = Compute_EWMA_Volatility(Dates,SPX,lambda);

proc (1) = Compute_Exposure(vol,vol_target,min_lev,max_lev);
  local lev;

  lev = vol_target/vol;
  lev = _min_(_max_(lev,min_lev),max_lev);

  retp(lev);
endp;

w = Compute_Exposure(sigma,0.20,0.0,1.0);
R_star = w .* R_SPX + (1 - w) .* R_Libor;
R_star[1] = 0.0;
S1 = 100 * cumprodc(1 + R_star);

w = Compute_Exposure(sigma,0.20,0.5,1.5);
R_star = w .* R_SPX + (1 - w) .* R_Libor;
R_star[1] = 0.0;
S2 = 100 * cumprodc(1 + R_star);

S = SPX~S1~S2;
Results = zeros(4,3);
i = 1;
do until i > 3;
  report = Compute_Global_Reporting(Dates,S[.,i],SPX,Libor,0,0);
  Results[.,i] = 100*vread(report,"mu") |
                 100*vread(report,"sigma") |
                 vread(report,"SR") |
                 vread(report,"mDD") ;
  i = i + 1;
endo;

print ftosa(Results,"%lf",2,2);

graphset;
  window3u;

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.28;
  _plwidth = 8; _pltype = 6|3;
  
  setwind(1);
    _pcolor = 1;
    title("\216\202s\201]t[");   
    xtics(0,nDates,261*5,2);
    let xlab = "90" "95" "00" "05" "10";
    asclabel(xlab,0);
    ytics(0,80,20,2);
    xy(0,100*sigma);

  ytics(0,800,200,2);

  setwind(2);
    title("\216(w]_[,w]+[) = (0,1)");
    _pcolor = 4;
    xy(0,S1);

  setwind(3);
    title("\216(w]_[,w]+[) = (0.5,1.5)");
    _pcolor = 4;
    xy(0,S2);

  graphprt("-c=1 -cf=stoploss5.eps");

  endwind;


   