/* 
** Classification
*/

new;
library qam,nnet,optmum,pgraph,gwizard;

rndseed 123;

estimate = 1;  // Change this value

let mu1 = 2 6;
let mu2 = 6 6;
let mu3 = 6 2;
let SIGMA[2,2] = 2 1 1 2;

N = 50;
x1 = rndmn(mu1,SIGMA,N);
x2 = rndmn(mu2,SIGMA,N);
x3 = rndmn(mu3,SIGMA,N);

value = 0|0.5|1;

t1 = value[1]*ones(N,1);
t2 = value[2]*ones(N,1);
t3 = value[3]*ones(N,1);

x = x1|x2|x3;
t = t1|t2|t3;

nnet_set;

_nnet_function = {1,1};
_nnet_DirectLink = 1;
_nnet_Constant = {1,1};
_nnet_PrintIters = 10;
_nnet_MaxIters = 2000;
_nnet_SaveIters = 1;
_nnet_flush = 0;
_nnet_StopRule = {1,0.999999};
_nnet_opalgr = 6;
_nnet_file = "ann11";

if estimate;
  nnet = nnet_fit(0,X,T,0,0,0,0,3);
  save ^_nnet_file = nnet;
else;
  loadm nnet = ^_nnet_file;
endif;

z = nnet_predict(nnet,x);

symbol = 1|9|11;
z = target_2_symbol(z);
t = target_2_symbol(t);

q = seqa(50,1,50);

indx = selif(seqa(1,1,3*N),z ./= t);

rndseed 123;

graphset;
  begwind;
  makewind(9/2,6.855/2,0,6.855/2,1);
  makewind(9/2,6.855/2,9/2,6.855/2,1);
  makewind(9/2,6.855/2,9/4,0,1);

    _pnum = 2; _paxht = 0.25; _pnumht = 0.20; _ptitlht = 0.25; _plwidth = 10;
    xtics(0,10,2,2);
    ytics(0,10,2,2);
    _plctrl = -1;
    _pstype = symbol; _psymsiz = 15;
    xlabel("\216X]1[");
    ylabel("\216X]2[");
  
  setwind(1);
    _pcolor = 2;
    title("\216Observation");
    _psym = x~t~ones(3*N,1).*(4~1~1~0);
    e = ones(rows(indx),1);
    _pline = e.*(4~6)~x[indx,.]~e.*(0.2~0~6.28~12~10);
    xy(q,q);

  setwind(2);
    title("\216Prevision");
    _psym = x~z~ones(3*N,1).*(4~2~1~0);
    _pmsgstr = "-";
    _pmsgctl = 4.57~6.42~0.07~45~2~0~6 ;

    xy(q,q);

    _pline = 0;
    _plctrl = 0; 
    _psym = 0;
    _pmsgctl = 0;

  setwind(3);

    N_sim = 5000;
    _plwidth = 10; _pltype = 6; _pcolor = 1;

    x_sim = rndu(N_sim,1)*10~3+rndu(N_sim,1)*2;
    x_sim = x_sim | 3+rndu(N_sim,1)*2~rndu(N_sim,1)*10;
    z_sim = target_2_symbol(nnet_predict(nnet,x_sim));

    x1 = CHULLPOINT(selif(x_sim,z_sim .== symbol[1]));
    x1 = trimr(delif(x1,x1[.,2] - x1[.,1] .> 5),3,0);
    x3 = CHULLPOINT(selif(x_sim,z_sim .== symbol[3]));
    x3 = delif(x3,x3[.,2] - x3[.,1] .< -5);
    x3 = rotater(x3',6)';

    max = maxc(rows(x1)|rows(x3));
    data_x = miss(zeros(max,2),0);
    data_y = miss(zeros(max,2),0);
    data_x[1:rows(x1),1] = x1[.,1];
    data_y[1:rows(x1),1] = x1[.,2];
    data_x[1:rows(x3),2] = x3[.,1];
    data_y[1:rows(x3),2] = x3[.,2];    
    
    title("\216Classe");
    _pmsgstr = "C]1[\000C]2[\000C]3[";
    _pmsgctl = {2 6 0.5 0 1 4 10,6 6 0.5 0 1 4 10,6 2 0.5 0 1 4 10};
    xy(data_x,data_y);

    graphprt("-c=1 -cf=ann11.eps");

  endwind;



proc target_2_symbol(z);
  local value;
  local e1,e2,e3,e;

  value = seqa(1,1,3)/3;
  e1 = z .lt value[1];
  e2 = z .ge value[1] .and z .lt value[2];
  e3 = z .ge value[2];
  e = e1~e2~e3;

  z = recode(z,e,symbol);
  retp(z);
endp;


proc CHULLPOINT(x);

   local n,s,imin,xmin1,xmin2,xmin3,xmin,xmax,xmax1,xmax2,xmax3,
   rmin,rmax,j,jmin,jmax,j1,j2,emin,emax;

   n = rows(x);
   s = seqa(1,1,n);
   imin = minindc(x[.,2]);

   j=1;
   j1=imin;
   j2=imin;
   do until j==0;                         /* j: row of x */
      xmin=x[j1,.];
      xmax=x[j2,.];
      xmin1=x-xmin;
      xmax1=x-xmax;
      xmin2=delif(xmin1~s, (sumc( (xmin1.==0)' ).>1));
      xmax2=delif(xmax1~s, (sumc( (xmax1.==0)' ).>1));
      xmin3=trimr(xmin2',0,1)';
      xmax3=trimr(xmax2',0,1)';
      emin=(xmin3[.,2].<0);      /* lower quadrants */
      emax=(xmax3[.,2].>0);
      rmin= arccos( xmin3[.,1]./sqrt(sumc(xmin3'^2)) );
      rmax= arccos( -xmax3[.,1]./sqrt(sumc(xmax3'^2)) );
      rmin=rmin+emin.*(2*pi-2*rmin);
      rmax=rmax+emax.*(2*pi-2*rmax);
      jmin=minindc(rmin);
      jmax=maxindc(rmax);
      j1=xmin2[jmin,3];
      j2=xmax2[jmax,3];
      if (j1==j2) or (j1==imin[1]);
          j=0;
          imin=imin|j1;
      else;

          if sumc(j1.==imin)>0;
              imin=j2|imin;
          elseif sumc(j2.==imin)>0;
              imin=imin|j1;
          else;
              imin=j2|imin|j1;
          endif;

      endif;
   endo;

   retp(x[imin,.]);
endp;
