new;
library qam,pgraph,optmum;

cls;

#include ir.params

_Bond_Continuous_Time = 0;

nt = 2;
t = seqa(1,1,nt);
Ct = 5 * ones(nt,1);
Ct[nt] = 100 + Ct[nt];
t1 = t; C1 = Ct;

nt = 5;
t = seqa(1,1,nt);
Ct = 5 * ones(nt,1);
Ct[nt] = 100 + Ct[nt];
t2 = t; C2 = Ct;

nt = 10;
t = seqa(1,1,nt);
Ct = 5 * ones(nt,1);
Ct[nt] = 100 + Ct[nt];
t3 = t; C3 = Ct;

fn Bond_Price(t,Ct,params) = Bond_Compute_Price(t,Ct,NelsonSiegel_SpotRate(params,t));

t = t1; Ct = C1;
P = Bond_Price(t,Ct,params);
YTM = Bond_Compute_YTM(t,Ct,P);
D = Bond_Compute_Duration(t,Ct,YTM);
P1 = P; YTM1 = YTM; D1 = D;

t = t2; Ct = C2;
P = Bond_Price(t,Ct,params);
YTM = Bond_Compute_YTM(t,Ct,P);
D = Bond_Compute_Duration(t,Ct,YTM);
P2 = P; YTM2 = YTM; D2 = D;

t = t3; Ct = C3;
P = Bond_Price(t,Ct,params);
YTM = Bond_Compute_YTM(t,Ct,P);
D = Bond_Compute_Duration(t,Ct,YTM);
P3 = P; YTM3 = YTM; D3 = D;

data = P1~100*YTM1~D1 |
       P2~100*YTM2~D2 |
       P3~100*YTM3~D3 ;

print ftosa(data,"%lf",3,2);

DV01 = D1|D2|D3;
Tau = 2|5|10;
Price = P1|P2|P3;

beta = 0.5;

{n1,n2,n3,n4} = Barbell_Calibrate(DV01,Price,Tau,beta);
n1 = 100*n1; n2 = 100*n2; n3 = 100*n3; n4 = 100*n4;
data = n1~n2~n3~n4;
print ftosa(data,"%lf",3,1);

t = seqa(1,1,10);
Rt = NelsonSiegel_SpotRate(params,t);

dR = -seqa(-20,1,41)/1000;
m = rows(dR);
PnL = zeros(m,4);

V0 = sumc(data.*Price);

j = 1;
do until j > m;
  n = n1;
  PnL[j,1] = Barbell_PnL(n[1],t1,C1,n[2],t2,C2,n[3],t3,C3,t,Rt,Rt+dR[j]);
  n = n2;
  PnL[j,2] = Barbell_PnL(n[1],t1,C1,n[2],t2,C2,n[3],t3,C3,t,Rt,Rt+dR[j]);
  n = n3;
  PnL[j,3] = Barbell_PnL(n[1],t1,C1,n[2],t2,C2,n[3],t3,C3,t,Rt,Rt+dR[j]);
  n = n4;
  PnL[j,4] = Barbell_PnL(n[1],t1,C1,n[2],t2,C2,n[3],t3,C3,t,Rt,Rt+dR[j]);
  j = j + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1; 
  _pltype = 6|1|3|6; _plwidth = 10; _pcolor = 1|3|4|2;
  _plegstr = "50/50\000Cash-neutral\000Maturity-weighted\000Regression-weighted";
  _plegctl = {2 5 5 0.5};
  xlabel("\216\202D\201]R[");
  ylabel("\216PnL");  
  xtics(-200,200,50,2);
  ytics(-10,20,5,2);
  graphprt("-c=1 -cf=ir21.eps");
  xy(1e4*dR,PnL);

proc (1) = Barbell_PnL(n1,t1,C1,n2,t2,C2,n3,t3,C3,t,Rt,new_Rt);
  local indx,R1,new_R1,R2,new_R2,R3,new_R3;
  local P1,new_P1,YTM1,D1,P2,new_P2,YTM2,D2,P3,new_P3,YTM3,D3;
  local PnL;

  indx = indnv(t1,t);
  R1 = Rt[indx];
  new_R1 = new_Rt[indx];

  indx = indnv(t2,t);
  R2 = Rt[indx];
  new_R2 = new_Rt[indx];

  indx = indnv(t3,t);
  R3 = Rt[indx];
  new_R3 = new_Rt[indx];

  P1 = Bond_Compute_Price(t1,C1,R1);
  new_P1 = Bond_Compute_Price(t1,C1,new_R1);
  YTM1 = Bond_Compute_YTM(t1,C1,P1);
  D1 = Bond_Compute_Duration(t1,C1,YTM1);

  P2 = Bond_Compute_Price(t2,C2,R2);
  new_P2 = Bond_Compute_Price(t2,C2,new_R2);
  YTM2 = Bond_Compute_YTM(t2,C2,P2);
  D2 = Bond_Compute_Duration(t2,C2,YTM2);

  P3 = Bond_Compute_Price(t3,C3,R3);
  new_P3 = Bond_Compute_Price(t3,C3,new_R3);
  YTM3 = Bond_Compute_YTM(t3,C3,P3);
  D3 = Bond_Compute_Duration(t3,C3,YTM3);

  PnL = n1*(new_P1 - P1) + n2*(new_P2-P2) + n3*(new_P3-P3);

  retp(PnL);
endp;

