new;
library qam,cml,pgraph,optmum;
cls;

__output = 1;
_cml_Algorithm = 3;

Base_Currency = "USD";
let string Currency[8,1] = "AUD" "CAD" "EUR" "GBP" "JPY" "NOK" "NZD" "SEK";
Maturity = 1;

nCurrency = rows(Currency);
Params = zeros(5,nCurrency);

PathName = "..\\data\\";
databuf = 0;

iter = 1;
do until iter > nCurrency;
  datFileName = PathName $+ "qam_FX";
  data = loadd(datFileName);
  vnames = "" $+ getname(datFileName);

  selected_names = "date" $| Currency[iter] $| (Currency[iter] $+ "-VOL") $| (Currency[iter] $+ "-1W") $| "USD-1W";
  indx = indsav(selected_names,vnames);
  data = data[.,indx];

  data = selif(data, (data[.,1] .> 19941231) .and (data[.,1] .<= 20091231));
  
  Dates = Data[.,1];
  Spot = Data[.,2];
  ImpVol = 1e-2 * Data[.,3];  // ImpVol
  IR_Currency = 1e-2 * Data[.,4];
  IR_Base = 1e-2 * Data[.,5];
  Yield = ln(Spot) - ln(lagn(Spot,1));
  
  Dates = trimr(Dates,1,0);
  Spot = trimr(Spot,1,0);
  ImpVol = trimr(ImpVol,1,0);
  IR_Currency = trimr(IR_Currency,1,0);
  IR_Base = trimr(IR_Base,1,0);
  Yield = trimr(Yield,1,0);
  
  nDates = rows(Dates);
  
  /*
  **> AR-GARCH
  **
  */
  
  nInSample = nDates;
  p = 1;
  p_garch = 1;
  q_garch = 1;

  T = Dates[1:nInSample];
  Y = Yield[1:nInSample,.];
  X = ones(nInSample,1) ~ lagn(Y,1);
  X[1,2] = Y[1];

  {beta,stderr,Mcov,u} = regOLS(Y,X,0,0);
  sv = beta|stdc(u)^2|0.01*ones(p_garch+q_garch,1);
  {theta,cl,cov,logl} = regGarch(Y,X,p_Garch,q_Garch,sv);

  if _garch_retcode;
    errorlog "error : convergence in CML.";
    end;
  endif;

  Params[.,iter] = theta;

  h2 = _garch_h;

  const = theta[1];
  phi = theta[2];
  kappa = theta[3];
  delta = theta[4];
  alpha = theta[5];

  h2 = miss(zeros(nDates,1),0);
  h2[1:nInSample] = _garch_h;
  
  i = nInSample + 1;
  do until i > nDates;
    u = Yield[i] - const - phi*Yield[i-1];
    h2[i] = kappa + delta*h2[i-1] + alpha*u^2;
    i = i + 1;
  endo;
  
  nDays_in_Year = 260;
  tau = Maturity/nDays_in_Year;
  ShortVol = sqrt(nDays_in_Year)*sqrt(h2);
  
  /*
  **> Probability
  **
  */
  
  Straddle = BS_Straddle(Spot,Spot,ImpVol,tau,IR_Base-IR_Currency,IR_Base);
  
  Sp = Spot + Straddle;
  Sm = Spot - Straddle;
  
  dp = (ln(Sp) - ln(Spot)) ./ (sqrt(tau) * ShortVol);
  dm = (ln(Sm) - ln(Spot)) ./ (sqrt(tau) * ShortVol);
  Pr = cdfn(dp) - cdfn(dm);
  
  Payoff = Straddle_Payoff(lagn(Spot,-Maturity),Spot);
  PnL = (Straddle - payoff) ./ Straddle;

  if iter == 1;
    ALL_Dates = Dates;
  else;
    ALL_Dates = unique(ALL_Dates | Dates,1);
  endif;

  tmp = Dates~PnL~Pr;
  databuf = vput(databuf,tmp,Currency[iter]);

  iter = iter + 1;
endo;

nDates = rows(ALL_Dates);
Dates = ALL_Dates;
Pr = miss(zeros(nDates,nCurrency),0);
PnL = miss(zeros(nDates,nCurrency),0);

iter = 1;
do until iter > nCurrency;
  tmp = vread(databuf,Currency[iter]);
  indx = indnv(tmp[.,1],Dates);
  PnL[indx,iter] = tmp[.,2];
  Pr[indx,iter] = tmp[.,3];
  iter = iter + 1;
endo;

loadm PnL_Cone = volatility3;

Pr_Shrink = miss(zeros(nDates,nCurrency),0);
PnL_Shrink = miss(zeros(nDates,nCurrency),0);
maxShrink = 0;

i = 1;
do until i > nCurrency;
  {tmp_Pr,tmp_PnL,tmp_indx} = Closest_Points(Pr[.,i],PnL[.,i],0.0005);
  nShrink = rows(tmp_Pr);
  maxShrink = _max_(maxShrink,nShrink);
  Pr_Shrink[1:nShrink,i] = tmp_Pr;
  PnL_Shrink[1:nShrink,i] = tmp_PnL;
  i = i + 1;
endo;

Pr_Shrink = Pr_Shrink[1:maxShrink,.];
PnL_Shrink = PnL_Shrink[1:maxShrink,.];

pqgwin many;

graphset; 
  begwind;
  makewind(9,6.855,0,0,1);
  makewind(9,6.855,0,0,1);

   fonts("simplex simgrma");  
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _paxht = 0.18; _pnumht = 0.15; _ptitlht = 0.20;
  xlabel("\216Score");
  ylabel("\216PnL");
  ytics(-8,2,2,2);
  xtics(20,100,20,2);

  setwind(1);
    _plwidth = 5; _plctrl = -1; _pcolor = seqa(1,1,8);
    _pstype = seqa(1,1,7)|1;
    _plegstr = MakeMsgStr(upper(Currency));
    _plegctl = {2 5 7 1};
    xy(100*Pr_Shrink,PnL_Shrink);

  setwind(2);
    _plwidth = 10; _plctrl = 0; 
    _pcolor = 0|0; _pltype = 3;
    _plegctl = 0;
    xy(PnL_Cone[.,1],PnL_Cone[.,2 5]);

    graphprt("-c=1 -cf=volatility4-a.eps");

  endwind;

/*
**> Buy Strategy
**
*/

Thresholds = 0|0.50|0.65;
nThresh = rows(Thresholds);

PnL1 = zeros(nDates,nThresh);
PnL2 = zeros(nDates,nThresh);
Notional2 = zeros(nDates,nThresh);

i = 1;
do until i > nThresh;
  x = missex(PnL, Pr .<= Thresholds[i]);

  tmp = miss(sumr(x ./= error(0)),0);
  weight = missrv(1 ./ tmp,0);
  PnL1[.,i] = sumr(missrv(x,0)) .* weight;

  weight = 1;
  PnL2[.,i] = sumr(missrv(x,0)) .* weight;
  Notional2[.,i] = sumr(missrv(x,0) ./= 0.0);

  i = i + 1;
endo;

CumPnL1 = cumsumc(PnL1);
CumPnL2 = cumsumc(PnL2);

PnL = PnL1;
CumPnL = CumPnL1;
stats = zeros(3,nThresh);

i = 1;
do until i > nThresh;
  tmp = selif(PnL[.,i], PnL[.,i] ./= 0);
  stats[.,i] = meanc(tmp)|stdc(tmp)|minc(tmp);
  i = i + 1;
endo;

print ftosa(stats,"%lf",3,2);

PnL = packr(PnL2 ./ Notional2);
CumPnL = CumPnL2;
stats = zeros(3,nThresh);

i = 1;
do until i > nThresh;
  tmp = selif(PnL[.,i], PnL[.,i] ./= 0);
  stats[.,i] = meanc(tmp)|stdc(tmp)|minc(tmp);
  i = i + 1;
endo;

print ftosa(stats,"%lf",3,2);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pltype = 1|3|6; _pcolor = 1|2|4;

  t = seqa(1,1,nDates);
  xtics(0,15*262,262,2);
  let xlab = "95" "96" "97" "98" "99" "00" "01" "02" "03" "04" "05" "06" "07" "08" "09" "10";
  asclabel(xlab,0);

  ytics(0,1250,250,2);

  _plegstr = MakeMsgStr(ftosa(100*Thresholds,"s[+] = %lf",1,0) $+ "%");
  _plegctl = {2 4 2 4};

  graphprt("-c=1 -cf=volatility4-b.eps");

  xy(0,CumPnL);

