new;
library qam,tsm,cml,optmum,pgraph;

cls;

declare external matrix parameters;

datPathName = "..\\data\\";
datFileName = datPathName $+ "qam_sp500_monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
Dates = data[.,1]/1e6;

let string selected_Names = "AdjClose";
indx_Selected_Names = indsav(selected_Names,vnames);
data = data[.,indx_Selected_Names];

BeginDate = 19500101;
EndDate = 20091231;
data = selif(Dates~data, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = data[.,1]; data = data[.,2];
nDates = rows(Data);

Rdt = Data./lag1(Data)-1;
Rdt = packr(Rdt);
T = rows(Rdt);

p = 1; q = 1;
{theta_Garch,cl_Garch,cov_Garch,logl_Garch} = regGarch(Rdt,1,p,q,0);
h_Garch = error(0) | _garch_h;
res_Garch = error(0) | Rdt - theta_Garch[1];
res_Garch = res_Garch ./ h_Garch;

m_Rdt = meanc(Rdt);
Rdt_c = Rdt - m_Rdt;
y = ln(Rdt_c^2);

proc sv_ml(theta);
  local LogL;
  local T,c,R,Q,Z,d,H;
        
  c = 0; 
  T = theta[1];   
  R = 1;
  Q = theta[2]^2;
  Z = 1;
  d = theta[3];
  H = theta[4]^2;

  call SSM_build(Z,d,H,T,c,R,Q,0);
  call Kfiltering(y,0,1);
  LogL = KF_ml;
  retp(LogL);
endp;

sv = 0.1 * ones(4,1);
{theta,stderr,Mcov,LogL} = TD_ml(&sv_ml,sv);

c = 0; T = theta[1]; R = 1; Q = theta[2]^2; Z = 1; d = theta[3]; H = theta[4]^2;
call SSM_build(Z,d,H,T,c,R,Q,0);
call Kfiltering(y,0,1);

h_KF = KF_matrix(3);
{h_KS,P_KS} = KSmoothing;

res_KF = error(0) | (Rdt_c ./ exp(h_KF/2));
res_KS = error(0) | (Rdt_c ./ exp(h_KS/2));
residual = res_KF ~ res_KS ~ res_Garch;
residual = packr(residual);
sigma = stdc(residual);

x = seqa(-0.20,0.40/255,256);
{d1,F1} = regKernelDensity(residual[.,1],x);
{d2,F2} = regKernelDensity(residual[.,2],x);
d3 = pdfn(x/sigma[1])/sigma[1];
d = d1~d2~d3;

r_k = acf(residual[.,1],50,0)~acf(residual[.,2],50,0);
pr_k = pacf(residual[.,1],50,0)~acf(residual[.,2],50,0);
r2_k = acf(residual[.,1]^2,50,0)~acf(residual[.,2]^2,50,0);
pr2_k = pacf(residual[.,1]^2,50,0)~acf(residual[.,2]^2,50,0);

pqgwin many;

h_KF = error(0) | h_KF;
h_KS = error(0) | h_KS;

vol_Garch = sqrt(h_Garch) * sqrt(12); // sqrt(monthly) * sqrt(12)
vol_KF = exp(h_KF/2);  // Comment transformer en volatilit annuelle
vol_KS = exp(h_KS/2);  // Comment transformer en volatilit annuelle

graphset;
  window3d;

  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6|6; _pcolor = 9|12;
  _ptitlht = 0.25; _pnumht = 0.20; _paxht = 0.25;

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10";
  asclabel(xlab,0);

  ytics(0,2,0.5,2);

  setwind(1);
    _pcolor = 1;
    title("\216Filtrage de Kalman");
    xy(0,vol_KF);

  setwind(2);
    _pcolor = 3;
    title("\216Lissage de Kalman");
    xy(0,vol_KS);

  ytics(0,35,5,2);

  setwind(3);
    _pcolor = 4;
    title("\216GARCH(1,1)");
    xy(0,100*vol_Garch);

  graphprt("-c=1 -cf=sv1-1.eps");

 endwind;

graphset;
  begwind;  
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = 0; _pframe = 0; _pnum = 2;
  _pbartyp = 6~9; _pbarwid = 0.1;
  _plwidth = 8; _pltype = 6|1|3; _pcolor = 1|2|3;
  _paxht = 0.225;  _ptitlht = 0.25;  _pnumht = 0.20; 

  alpha = 0.05;

  z_alpha = cdfni(1-alpha/2);
  e = z_alpha/sqrt(nDates);
  _pline = 1~1~-10~e~60~e~1~2~10 |
           1~1~-10~-e~60~-e~1~2~10;    
  xlabel("\216k");

  ytics(-0.10,0.10,0.05,2);
  xtics(-5,55,5,2);
  let xlab = "0" "" "10" "" "20" "" "30" "" "40" "" "50" "";
  asclabel(xlab,0);
  _pbarwid = 0.4;

  setwind(1);
    _pbartyp = 6~1;
    title("\216ACF \202e\201]t[[*]");
    bar(0,r_k);

  setwind(2);
    _pbartyp = 6~3;
    title("\216ACF (\202e\201]t[[*])[2]");
    bar(0,r2_k);

  setwind(3);
    _pbartyp = 6~4;
    title("\216PACF (\202e\201]t[[*])[2]");
    bar(0,pr2_k);

  ylabel(""); xlabel(""); _pline = 0;

  setwind(4);
    title("\216PDF de \202e\201]t[[*]");
    ytics(0,12,2,2);
    xtics(-0.20,0.20,.05,1);
    _plegctl = {2 5 6 4};
    _plegstr = "KF\000KS\000Normal";
    xy(x,d);
    graphprt("-c=1 -cf=sv1-2.eps");

  endwind;