/*
**> Results MCMC
*/

new;
library qam,pgraph,optmum;

datPathName = "..\\data\\";
datFileName = datPathName $+ "qam_sp500_monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
Dates = data[.,1]/1e6;

let string selected_Names = "AdjClose";
indx_Selected_Names = indsav(selected_Names,vnames);
data = data[.,indx_Selected_Names];

BeginDate = 19500101;
EndDate = 20091231;

data = selif(Dates~data, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = data[.,1]; data = data[.,2];
nDates = rows(Data);

Rdt = Data./lag1(Data)-1;
Rdt = packr(Rdt);
T = rows(Rdt);

proc _Historical_Volatility(x,Period);
  local n,vol,i;

  n = rows(x);
  vol = miss(zeros(n,1),0);
  i = 1;
  do until i > n-Period;
    vol[i] = stdc(x[i:i+Period]);
    i = i + 1;
  endo;

  retp(vol);
endp;

hVol = _Historical_Volatility(Rdt,12);

let string FileNames = "sv2" "sv5" "sv3" "sv4";
nModels = rows(FileNames);

Nr = 3000;
Ns = 8000;

data = zeros(T,nModels);

i = 1;
do until i > nModels;
  FileName = FileNames[i];
  loadm all_results = ^FileName;

  h = vread(all_results,"h");
  h = h[.,Nr+1:Nr+Ns];
  sVol = exp(0.5 * h);
  sVol_mean = meanc(sVol');

  data[.,i] = sVol_mean;
  i = i + 1;
endo;

data = hVol~data;

data[.,1] = lagn(data[.,1],6);

data = 100*sqrt(12) * data;

graphset;
  begwind;
  window(2,2,0);

  _pdate = ""; _pframe = 0; _pnum = 2;
  _plwidth = 10|6; _pltype = 3|6; _pcolor = 9|12;
  _ptitlht = 0.25; _pnumht = 0.20; _paxht = 0.25;
  _pstype = 9; _plctrl = 0|24;

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10";
  asclabel(xlab,0);

  ytics(5,30,5,2);

  setwind(1);
    _pcolor = 1|4;
    title("\216Griddy Gibbs");
    xy(0,data[.,2~1]);

  setwind(2);
    _pcolor = 2|4;
    title("\216Metropolis-Hastings Griddy-Gibbs");
    xy(0,data[.,3~1]);

  setwind(3);
    _pcolor = 3|4;
    title("\216Random Walk Metropolis");
    xy(0,data[.,4~1]);

  setwind(4);
    _pcolor = 5|4;
    title("\216Metropolis-Hastings");
    _plegstr = "Stochastique\000Historique";
    _plegctl = {2 5 0.9 4.5};
    xy(0,data[.,5~1]);

    graphprt("-c=1 -cf=sv6.eps");

  endwind;