new;
library pf,pgraph,qam,optmum;

cls; 

/* 
**> Sample the importance density
*/

proc (1) =  pf_importance_sampling(t,x,z);

  x = pf_prior_sampling(t,x);

  retp(x);
endp;

/* 
**> Sample the prior density
*/

proc (1) =  pf_prior_sampling(t,x);
  local nS,P,u,Ft,Qt;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc, Q:proc;

  nS = rows(x);
  P = cols(x);
  u = rndn(Ns,P);

  Ft = F(t,x);
  Qt = Q(t,x);  

  x = Ft + sqrt(Qt) * u;

  retp(x);
endp;


/* 
**> Evaluate the importance density function: q(x_{t}|x{t-1},z_{t})
*/

proc (1) = pf_importance_density(t,nx,x,z);
  local pdf;

  pdf = pf_prior_density(t,nx,x);

  retp(pdf);
endp;

/* 
**> Evaluate the prior density function: p(x_{t}|x{t-1})
**
** Remark: It is the same as the importance density function
*/

proc (1) = pf_prior_density(t,nx,x);
  local Ft,Qt,pdf;

  local F,Q;
  F = _pf_ft;
  Q = _pf_Qt;
  local F:proc,Q:proc;

  Ft = F(t,x);
  Qt = Q(t,x);

  pdf = pdfn( (nx - Ft) ./ sqrt(Qt) ) ./ sqrt(Qt);

  retp(pdf);
endp;

/* 
**> Evaluate the likelihood function p(z_{t}|x_{t})
*/

proc (1) = pf_likelihood_density(t,nx,z);
  local Ht,Rt,pdf;

  local H,R;
  H = _pf_ht;
  R = _pf_Rt;
  local H:proc,R:proc;
  
  Ht = H(t,nx);
  Rt = R(t,nx);
  
  pdf = pdfn( (z - Ht) ./ sqrt(Rt) ) ./ sqrt(Rt);

  retp(pdf);
endp;

/* 
**> Sample the initial state vector
*/

proc (1) = pf_initial_sampling(nS);
  local x;
   x = 5*ones(nS,1);
  retp(x);
endp;

/*
**> State Space Model
*/

fn pf_ft(t,x) = x./2 + 25*x./(1 + x^2) + 8*cos(1.2*t);
fn pf_Qt(t,x) = 1;

fn pf_ht(t,x) = x^2 ./ 20;
fn pf_Rt(t,x) = 10;

/*
**> Main Test
**
*/

call Particle_Filter_Set(&pf_Ft,&pf_Qt,&pf_Ht,&pf_Rt,
                         &pf_importance_density,&pf_prior_density,&pf_likelihood_density,0,
                         &pf_importance_sampling,&pf_prior_sampling,&pf_initial_sampling);

/* Simulate the state and observed variables */

x0 = pf_initial_sampling(1);
nT = 100;
nR = 1;

//{t,xt,zt} = Simulate_Particle_Filter(x0,nT,nR);
nDates = nT;

/* Run the particle filter */

rndseed 123;

nS = 1000;  // Number of simulations
nRuns = 100;     // Number of runs of the MC
P = 1;      // Number of state variables

results_xt = zeros(nDates,nRuns);
results_zt = zeros(nDates,nRuns);

SIS_m = zeros(nDates,nRuns);
SIS_rmse = zeros(nRuns,1);

PF_m = zeros(nDates,nRuns);
PF_rmse = zeros(nRuns,1);

SIR_m = zeros(nDates,nRuns);
SIR_rmse = zeros(nRuns,1);

RPF_m = zeros(nDates,nRuns);
RPF_std = zeros(nDates,nRuns);
RPF_rmse = zeros(nRuns,1);

print "---------------- RMSE -----------------";
print "             SIS      PF     SIR     RPF";

i = 1;
do until i > nRuns;

  {t,xt,zt} = Simulate_Particle_Filter(x0,nT,nR);     /* Simulating a SSM */
  results_xt[.,i] = xt;
  results_zt[.,i] = zt;

  {x,w,m,cov} = SIS_Particle_Filter(zt,nS);                /* Running the SIS particle filter */ 
  SIS_m[.,i] = m;
  SIS_rmse[i] = sqrt(meanc((m - xt)^2))';   

  {x,w,m,cov} = Generic_Particle_Filter(zt,nS);                /* Running the generalized particle filter */     
  PF_m[.,i] = m;
  PF_rmse[i] = sqrt(meanc((m - xt)^2))';   

  {x,w,m,cov} = SIR_Particle_Filter(zt,nS);             /* Running the SIR particle filter */     
  SIR_m[.,i] = m;
  SIR_rmse[i] = sqrt(meanc((m - xt)^2))';   

  {x,w,m,cov} = Regularized_Particle_Filter(zt,nS);  /* Running the regularised particle filter */   
  RPF_m[.,i] = m;
  RPF_rmse[i] = sqrt(meanc((m - xt)^2))';

  str = ftos(i,"%f",8,0) $+
        ftos(SIS_rmse[i],"%f",8,3) $+
        ftos(PF_rmse[i],"%f",8,3) $+
        ftos(SIR_rmse[i],"%f",8,3) $+
        ftos(RPF_rmse[i],"%f",8,3) ;
  print /flush str;

  i = i + 1;   
endo; 

print "---------------------------------------";

str = "mean:   " $+
      ftos(meanc(SIS_rmse),"%f",8,3) $+
      ftos(meanc(PF_rmse),"%f",8,3) $+
      ftos(meanc(SIR_rmse),"%f",8,3) $+
      ftos(meanc(RPF_rmse),"%f",8,3) ;
print /flush str;

str = "std-err:" $+
      ftos(stdc(SIS_rmse),"%f",8,3) $+
      ftos(stdc(PF_rmse),"%f",8,3) $+
      ftos(stdc(SIR_rmse),"%f",8,3) $+
      ftos(stdc(RPF_rmse),"%f",8,3) ;
print /flush str;

run_indx = 1;
do until run_indx > nRuns;

  graphset;
    window3d;
      _pdate = 0; _pnum = 2; _pframe = 0;
      _pnumht = 0.20; _paxht = 0.20; _ptitlht = 0.25;
      _plwidth = 10; _pltype = 6|3|1;
      _pcolor = 1|2|3;
      xlabel("\216t");
      xtics(0,100,20,2);
      fonts("simplex simgrma");

    setwind(1);
      _pcolor = 1;
      title("\216\202a\201]t[");
      ytics(-20,20,10,2);
      xy(0,results_xt[.,run_indx]);

    setwind(2);
      _pcolor = 2;
      title("\216y]t[");
      ytics(-6,18,6,2);
      xy(0,results_zt[.,run_indx]);
  
    setwind(3);
      _pcolor = 4|5|6|7; _pltype = 6|3|1|5;
      _plegstr = "SIS\000PF\000SIR\000RPF"; 
      _plegctl = {2 5 9 4};
      title("\216Estimation de \202a\201]t[");
      ytics(-20,20,10,2);
      xy(0,SIS_m[.,run_indx]~PF_m[.,run_indx]~SIR_m[.,run_indx]~RPF_m[.,run_indx]);

    endwind;

  call pause(0.2);

  run_indx = run_indx + 1;
endo;

pqgwin many;

data  = SIS_rmse~PF_rmse~SIR_rmse~RPF_rmse;
x_rmse = seqa(0,0.1,101);
d_rmse = zeros(rows(x_rmse),4);
i = 1;
do until i > 4;
  {d_rmse[.,i],F_rmse} = regKernelDensity(Data[.,i],x_rmse);
  i = i + 1;
endo;

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcolor = 1|2|3|4; _pltype = 6|3|1|5;
  _plwidth = 10;
  xlabel("\216RMSE");
  xtics(0,10,1,2);
  _plegstr = "SIS\000GPF\000SIR\000RPF"; 
  _plegctl = {2 5 7 5};
  graphprt("-c=1 -cf=ts20-2.eps");    
  xy(x_rmse,d_rmse);

run_indx = nRuns;

graphset;
  window3d;
    _pdate = 0; _pnum = 2; _pframe = 0;
    _pnumht = 0.225; _paxht = 0.20; _ptitlht = 0.275;
    _plwidth = 10; _pltype = 6|3|1;
    _pcolor = 1|2|3;
    xlabel("\216t");
    xtics(0,100,20,2);
    fonts("simplex simgrma");

  setwind(1);
    _pcolor = 1;
    title("\216\202a\201]t[");
    ytics(-20,20,10,2);
    xy(0,results_xt[.,run_indx]);

  setwind(2);
    _pcolor = 2;
    title("\216y]t[");
    ytics(-6,18,6,2);
    xy(0,results_zt[.,run_indx]);

  setwind(3);
    _pcolor = 4|5|6|7; _pltype = 6|3|1|5;
    _plegstr = "SIS\000PF\000SIR\000RPF"; 
    _plegctl = {2 5 9 4};
    title("\216Estimation de \202a\201]t[");
    ytics(-20,20,10,2);
    xy(0,SIS_m[.,run_indx]~PF_m[.,run_indx]~SIR_m[.,run_indx]~RPF_m[.,run_indx]);

    graphprt("-c=1 -cf=ts20-1.eps");    

  endwind;

