new;
library optmum,pgraph,qam;
cls;

/*
** Models with conditional volatility
**
*/ 

datPathName = "..\\data\\";
datFileName = datPathName $+ "qam_sp500_monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
Dates = data[.,1]/1e6;

let string selected_Names = "AdjClose";
indx_Selected_Names = indsav(selected_Names,vnames);
data = data[.,indx_Selected_Names];

BeginDate = 19500101;
EndDate = 20091231;
data = selif(Dates~data, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = data[.,1]; data = data[.,2];
nDates = rows(Data);

Rdt = Data./lag1(Data)-1;
Rdt = packr(Rdt);
T = rows(Rdt);

/*
**> Ljung-Box test of randomness of residuals
**  H_0: "residuals are random" vs. H_1: "residuals are not random"
**
*/

alpha = 0.05;
s = 100;
r_k = acf(packr(Rdt^2),s,0);
Q_Ljung_Box = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s) );
Q_pvalue = cdfchic(Q_Ljung_Box, s);

print "***********************************";
print "*        Ljung-Box Test           *";
print "***********************************";

if Q_pvalue < alpha;
  msg = ftos(s,"The Ljung-Box test statistic for s = %lf is ",1,0) $+ ftos(Q_Ljung_Box,"%lf",3,2);
  print msg;
  msg = ftos(Q_pvalue,"The Ljung-Box test statistic's p-value is %lf",1,4);
  print msg; 
  msg = ftos(alpha,"At alpha =  %lf, we reject the null hypothesis of random residuals",1,2);
  print msg;
else;
  msg = ftos(s, "The Ljung-Box test statistic for the first %*.*lf lags is ",1,0) $+ ftos(Q_Ljung_Box,"%lf",3,2);
  print msg;
  msg = ftos(Q_pvalue, "The Ljung-Box test statistic's p-value is %*.*lf",1,4);
  print msg; 
  msg = ftos(alpha, "At alpha =  %*.*lf, we fail to reject the null hypothesis of random residuals",1,2);
  print msg; 
endif;

s = 50;
r_k = acf(Rdt,s,0);
r2_k = acf(Rdt^2,s,0);
pr2_k = pacf(Rdt^2,s,0);

graphset;
  window3u;
  makewind(9/2,6.855/2,9/4,6.855/2,1);
  makewind(9/2,6.855/2,0,0,1);
  makewind(9/2,6.855/2,9/2,0,1);
  
  _pdate = 0; _pframe = 0; _pnum = 2;   _pbarwid = 0.4;
  _paxht = .25;  _ptitlht = .275;  _pnumht = .20; 

  ytics(-0.10,0.15,0.05,2);
  xtics(-5,55,5,2);
  let xlab = "0" "" "10" "" "20" "" "30" "" "40" "" "50" "";
  asclabel(xlab,0); 
 
  z_alpha = cdfni(1-alpha/2);
  e = z_alpha/sqrt(T);
  _pline = 1~1~-10~e~60~e~1~2~10 |
           1~1~-10~-e~60~-e~1~2~10;    

  setwind(1);
    title("\216ACF r]t[");
    xlabel("\216k");
    _pbartyp = 6~1; 
    bar(0,r_k);

  setwind(2);
    title("\216ACF r]t[[2]");
    _pbartyp = 6~3; 
    bar(0,r2_k);

  setwind(3);
    title("\216PACF r]t[[2]");
    _pbartyp = 6~4; 
    bar(0,pr2_k);

    graphprt("-c=1 -cf=ts31.eps");

  endwind;
