new;
library qam,pgraph,optmum,cml;
cls;

/*
** Estimation of a GARCH model on S&P 500 monthly returns
**
*/

datPathName = "..\\data\\";
datFileName = datPathName $+ "qam_sp500_monthly";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
Dates = data[.,1]/1e6;

let string selected_Names = "AdjClose";
indx_Selected_Names = indsav(selected_Names,vnames);
data = data[.,indx_Selected_Names];

BeginDate = 19500101;
EndDate = 20091231;
data = selif(Dates~data, (Dates .>= BeginDate) .and (Dates .<= EndDate));
Dates = data[.,1]; data = data[.,2];
nDates = rows(Data);

Rdt = Data./lag1(Data)-1;
Rdt = packr(Rdt);
T = rows(Rdt);

// Garch ML
p = 1; q = 1;
{theta,cl,cov,logl} = regGarch(Rdt,1,p,q,0);

h = error(0) | _garch_h;
u = error(0) | (Rdt - theta[1]);
u = u ./ h;

// t-test of coefficients
stderr = sqrt(diag(cov));
tstudent = theta ./ stderr;
df = T - rows(theta);
pvalue = 2*cdftc(abs(tstudent),df);
alpha = 0.05;

cls;

print "***********************************";
print "*        Coeffs T-Tests           *";
print "***********************************";
format /mat /m1 /lds 8,6;
format /str /m1 /lds 12,0;

let string msg = "coeff" "t-stat" "p-value";
print msg';
msg =  "CONST" | 
        "kappa" |
        (0 $+ "delta" $+ ftocv(seqa(1,1,1),1,0)) |
        (0 $+ "alpha" $+ ftocv(seqa(1,1,q),1,0)) ;
msg = msg~theta~tstudent~pvalue;
call printfmt(msg,0~1~1~1);
print; print;

/*
**> Ljung-Box test of randomness of residuals
**  H_0: "residuals are random" vs. H_1: "residuals are not random"
**
*/

s = 5;
r_k = acf(packr(u),s,0);
pr_k = pacf(packr(u),s,0);
Q_Ljung_Box = T*(T+2)*sumc(r_k^2 ./ seqa(T,-1,s) );
Q_pvalue = cdfchic(Q_Ljung_Box,s);

print "***********************************";
print "*        Ljung-Box Test           *";
print "***********************************";

if Q_pvalue < alpha;
  msg = ftos(s,"The Ljung-Box test statistic for s = %lf is ",1,0) $+ ftos(Q_Ljung_Box,"%lf",3,2);
  print msg;
  msg = ftos(Q_pvalue,"The Ljung-Box test statistic's p-value is %lf",1,4);
  print msg; 
  msg = ftos(alpha,"At alpha =  %lf, we reject the null hypothesis of random residuals",1,2);
  print msg;
else;
  msg = ftos(s, "The Ljung-Box test statistic for the first %*.*lf lags is ",1,0) $+ ftos(Q_Ljung_Box,"%lf",3,2);
  print msg;
  msg = ftos(Q_pvalue, "The Ljung-Box test statistic's p-value is %*.*lf",1,4);
  print msg; 
  msg = ftos(alpha, "At alpha =  %*.*lf, we fail to reject the null hypothesis of random residuals",1,2);
  print msg; 
endif;

Q_pvalue = miss(zeros(6,1),0);
Q_Ljung_Box = miss(zeros(6,1),0);
s = seqa(1,1,6);
i = 1;
do until i > 6;
  r_k = acf(packr(u),s[i],0);
  pr_k = pacf(packr(u),s[i],0);
  Q_Ljung_Box[i] = T * (T+2) * sumc(r_k^2 ./ seqa(T,-1,s[i]) );
  Q_pvalue[i] = cdfchic(Q_Ljung_Box[i], s[i]);
  i = i + 1;  
endo;

print ftosa(Q_Ljung_Box'|Q_pvalue',"%lf",3,3);

s = 50;
r_k = acf(packr(u),s,0);
pr_k = pacf(packr(u),s,0);
vol = 100*stdc(Rdt)*sqrt(12);

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");

  _pdate = 0; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6; _pcolor = 1;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20; 

  xtics(1,nDates,12*10,1);
  let xlab = "50" "60" "70" "80" "90" "2000" "10";
  asclabel(xlab,0);

  _pcolor = 1;
  setwind(1);
    title("\216\202s\201]t[ (en %)");
    ytics(0,35,5,2);
    _pline = 1~3~0~vol~1000~vol~1~5~10;
    xy(0,100*sqrt(h)*sqrt(12));

  _pcolor = 2;
  nextwind;
    title("\216\202e\201]t[[*]");
    ytics(-100,100,50,2);
    xy(0,u);

  z_alpha = cdfni(1-alpha/2);
  e = z_alpha/sqrt(T);
  _pline = 1~1~-10~e~60~e~1~2~10 |
           1~1~-10~-e~60~-e~1~2~10;    
  xlabel("\216k");

  ytics(-0.10,0.10,0.05,2);
  xtics(-5,55,5,2);
  let xlab = "0" "" "10" "" "20" "" "30" "" "40" "" "50" "";
  asclabel(xlab,0);
   _pbarwid = 0.4;  

  nextwind;
    _pbartyp = 6~3;
    title("\216ACF \202e\201]t[[*]");
    bar(0,r_k);

  nextwind;
    _pbartyp = 6~4;
    title("\216PACF \202e\201]t[[*]");
    bar(0,pr_k);

  graphprt("-c=1 -cf=ts32.eps");

  endwind;