/*
** Donoho and Johnstone [1994], Ideal spatial adaptation by wavelet shrinkage,
** Biometrika, 81, 425-455
*/

new;
library tsm,optmum,pgraph;

rndseed 123456;

proc sgn(x);
  local s;
  s = 2*(x.>0)-1;
  retp(s);
endp;

proc K1(t);
  retp(0.5+0.5*sgn(t));
endp;

proc K2(t);
  retp((1+abs(t))^(-4));
endp;

epsilon = 0.05;

N = 2048;
t = seqa(0,1/(N-1),N);

t1 = 0.1|0.13|0.15|0.23|0.25|0.40|0.44|0.65|0.76|0.78|0.81;
h1 = 4|-5|3|-4|5|-4.2|2.1|4.3|-3.1|2.1|-4.2;
t2 = t1;
h2 = 4|5|3|4|5|4.2|2.1|4.3|3.1|5.1|4.2;
w2 = 5|6|6|10|10|30|10|10|5|8|5; w2 = w2/1000;


Blocks = 4*sumc(h1.*K1(t'-t1));
Bumps = 10*sumc(h2.*K2((t'-t2)./w2));
HeaviSine = 2.5*4*sin(4*pi*t) - sgn(t-0.3) - sgn(0.72-t);
Doppler = 10*sqrt(t.*(1-t)).*sin(2*pi*(1+epsilon)./(t+epsilon));

Data = Blocks~Bumps~HeaviSine~Doppler;

sc = stdc(Blocks)/14;
Noisy_Blocks = Blocks + rndn(N,1)*sc;
sc = stdc(Bumps)/14;
Noisy_Bumps = Bumps + rndn(N,1)*sc;
sc = stdc(HeaviSine)/14;
Noisy_HeaviSine = HeaviSine + rndn(N,1)*sc;
sc = stdc(Doppler)/14;
Noisy_Doppler = Doppler + rndn(N,1)*sc;

{H,G,Htilde,Gtilde} = Daubechies(4);

_wcenter = 0;
Noisy_Data = Noisy_Blocks~Noisy_Bumps~Noisy_HeaviSine~Noisy_Doppler;
x = Noisy_Data;
y = zeros(N,4);

i = 1;
do until i>4;

  w = wt(x[.,i],H,G,0);

  j = 1;
  do until j > 5;
    bnd = select(w,j);
    bnd = VisuShrink(bnd,"Hard");
    w = insert(w,bnd,j);
    j = j + 1;
  endo;

  y[.,i] = iwt(w,Htilde,Gtilde,0);

  i = i + 1;
endo;

all_data = data~Noisy_Data~y;

let string str[4,1] = "Blacks" "Bumps" "HeaviSine" "Doppler";

graphset;
  begwind;
  window(3,4,0);

  _pdate = ""; _pnum = 2; _pframe = 0;
  _paxht = 0.20; _pnumht = 0.30; _ptitlht = 0.45;
  _plwidth = 8;

  xtics(0,1,0.2,2);

  i = 1;
  do until i > 12;
    setwind(i);
      indx = i % 4;
      if indx == 0;
        indx = 4;
      endif;
      _pcolor = indx;
      if indx == 1;
        ytics(-10,25,5,2);
      elseif indx == 2;
        ytics(0,50,10,2);
      elseif indx == 3;
        ytics(-15,15,5,2);
      elseif indx == 4;
        ytics(-6,6,3,2);
      endif;
      if i <= 4;
        title("\216" $+ str[i]);
      else;
        title("");
      endif;
      xy(t,all_Data[.,i]);
    i = i + 1;
  endo;

  graphprt("-c=1 -cf=wavelet10.eps");

  endwind;

