/*
**  Wornell and Oppenheim [1992], Estimation of fractal signals
**  from noisy measurements using wavelets,
**  IEEE transactions on Signals Processing, 40, 611-623
*/

new;
library qam,tsm,optmum,pgraph;
TSMset;

M = 9;
Nobs = 2^M;
Nr = 1000;

let d = 0 0.25 -0.25 0.45;

Estimates = zeros(Nr,2*4);

i = 1;
do until i > 4;

  if i == 1;
    data = rndn(Nobs,Nr)*2;
  else;
    {data,retcode} = RND_arfima(d[i],0,0,2,2000,Nobs,Nr);
  endif;

  Nbsimul = 1;
  do until Nbsimul > Nr;

    x = data[.,Nbsimul];

    /* GPH Method */
    d1 = GPH(x);

    /* Wavelet Method */
    d2 = WaveletMethod(x);

    Estimates[Nbsimul,2*(i-1)+1:2*i] = d1~d2;
    Nbsimul = Nbsimul + 1;

  endo;

  i = i + 1;
endo;

x = seqa(-1.0,2/500,501);
pdf = zeros(rows(x),8);

j = 1;
do until j > 8;
  {pdf[.,j],cdf} = regKernelDensity(Estimates[.,j],x);
  j = j + 1;
endo;

graphset;  
  begwind;
  window(2,2,0);

    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6|3; _plwidth = 10; _pcolor = 1|3;
    _paxht = 0.225; _pnumht = 0.25; _ptitlht = 0.30;
    xtics(-1.0,1.0,0.5,2);
    ytics(0,12,2,2);
    xlabel("\216d"); 

  i = 1;
  do until i > 4;
    setwind(i);
      if i == 1;
        title("\216d = 0");
      elseif i == 2;
        title("\216d = 0,25");
      elseif i == 3;
        title("\216d = -0,25");
      elseif i == 4;
        title("\216d = 0,45");
     endif;

      str = ftos(d[i],"d = %lf",1,2);
      if i == 4;
        _plegstr = "GPH\000Ondelettes";
        _plegctl = {2 5.75 1.5 4};
      endif;
      xy(x,pdf[.,2*(i-1)+1:2*i]);
    i = i + 1;
  endo;

  graphprt("-c=1 -cf=wavelet13.eps");

  endwind;


/*
** Procedure for the GPH method
*/

proc (1) = gph(x);
  local lambda,I,Nobs,M,Y,w,XX,beta,d;

  _fourier = 0;
  {lambda,I} = PDGM(x);

  Nobs = rows(I);
  M = trunc(sqrt(Nobs));

  Y = ln(I[2:M+1]);
  w = 4*sin(lambda[2:M+1]/2)^2;
  XX = ones(M,1)~-ln(w);

  beta = Y/XX;
  d = beta[2];

  retp(d);
endp;


/*
** Procedure for the Wavelet Method
*/

declare matrix _wcoeff, _scale;

proc ml(theta);
  local sigma,d,w,j,logl,i,u,v;
  sigma = theta[1];
  d = theta[2];
  w = _wcoeff;
  logl = {};
  j = 1;
  do until j > rows(_scale);
    i = _scale[j];
    u = select(w,i);
    v = (sigma^2)*2^(2*d*i);
    logl = logl | (-0.5*ln(2*pi)-0.5*ln(v)-0.5*u^2/v);
    j = j + 1;
  endo;

  retp(logl);
endp;

proc WaveletMethod(x);
  local H,G,Htilde,Gtilde,w;
  local sv,theta,stderr,Mcov,Logl;

  M = ln(rows(x))/ln(2);

  {H,G,Htilde,Gtilde} = Daubechies(4);

  w = wt(x,H,G,0);
  _wcoeff = w;
  _scale = seqa(1,1,M);
  sv = 4|0;

  __output = 0;
  _print = 0;
  _tsm_Mcov = 0;

  {theta,stderr,Mcov,Logl} = TD_ml(&ml,sv);

  retp(theta[2]);
endp;

